@@ -1,11 +1,11 @@
 from mlflow.anthropic.autolog import async_patched_class_call, patched_class_call
-from mlflow.utils.annotations import experimental
+from mlflow.telemetry.events import AutologgingEvent
+from mlflow.telemetry.track import _record_event
 from mlflow.utils.autologging_utils import autologging_integration, safe_patch
 
 FLAVOR_NAME = "anthropic"
 
 
-@experimental
 @autologging_integration(FLAVOR_NAME)
 def autolog(
     log_traces: bool = True,
@@ -14,7 +14,7 @@ def autolog(
 ):
     """
     Enables (or disables) and configures autologging from Anthropic to MLflow.
-    Only synchronous calls and asynchnorous APIs are supported. Streaming is not recorded.
+    Only synchronous calls and asynchronous APIs are supported. Streaming is not recorded.
 
     Args:
         log_traces: If ``True``, traces are logged for Anthropic models.
@@ -38,3 +38,7 @@ def autolog(
         "create",
         async_patched_class_call,
     )
+
+    _record_event(
+        AutologgingEvent, {"flavor": FLAVOR_NAME, "log_traces": log_traces, "disable": disable}
+    )
