@@ -2,15 +2,19 @@ import base64
 import json
 import logging
 from functools import lru_cache
-from typing import Any, Optional, Union
+from typing import Any, Union
 
 from google.protobuf.json_format import MessageToDict, ParseDict
 from google.protobuf.struct_pb2 import Value
+from opentelemetry.proto.trace.v1.trace_pb2 import Span as OTelProtoSpan
+from opentelemetry.proto.trace.v1.trace_pb2 import Status as OTelProtoStatus
 from opentelemetry.sdk.resources import Resource as _OTelResource
 from opentelemetry.sdk.trace import Event as OTelEvent
 from opentelemetry.sdk.trace import ReadableSpan as OTelReadableSpan
 from opentelemetry.trace import NonRecordingSpan, SpanContext, TraceFlags
 from opentelemetry.trace import Span as OTelSpan
+from opentelemetry.trace import Status as OTelStatus
+from opentelemetry.trace import StatusCode as OTelStatusCode
 
 import mlflow
 from mlflow.entities.span_event import SpanEvent
@@ -25,7 +29,14 @@ from mlflow.tracing.utils import (
     decode_id,
     encode_span_id,
     encode_trace_id,
+    generate_mlflow_trace_id_from_otel_trace_id,
 )
+from mlflow.tracing.utils.otlp import (
+    _decode_otel_proto_anyvalue,
+    _otel_proto_bytes_to_id,
+    _set_otel_proto_anyvalue,
+)
+from mlflow.tracing.utils.processor import apply_span_processors
 
 _logger = logging.getLogger(__name__)
 
@@ -45,11 +56,12 @@ class SpanType:
     PARSER = "PARSER"
     EMBEDDING = "EMBEDDING"
     RERANKER = "RERANKER"
+    MEMORY = "MEMORY"
     UNKNOWN = "UNKNOWN"
 
 
 def create_mlflow_span(
-    otel_span: Any, trace_id: str, span_type: Optional[str] = None
+    otel_span: Any, trace_id: str, span_type: str | None = None
 ) -> Union["Span", "LiveSpan", "NoOpSpan"]:
     """
     Factory function to create a span object.
@@ -123,12 +135,12 @@ class Span:
         return self._span._start_time
 
     @property
-    def end_time_ns(self) -> Optional[int]:
+    def end_time_ns(self) -> int | None:
         """The end time of the span in nanosecond."""
         return self._span._end_time
 
     @property
-    def parent_id(self) -> Optional[str]:
+    def parent_id(self) -> str | None:
         """The span ID of the parent span."""
         if self._span.parent is None:
             return None
@@ -197,7 +209,7 @@ class Span:
             f"span_id={self.span_id!r}, parent_id={self.parent_id!r})"
         )
 
-    def get_attribute(self, key: str) -> Optional[Any]:
+    def get_attribute(self, key: str) -> Any | None:
         """
         Get a single attribute value from the span.
 
@@ -338,8 +350,92 @@ class Span:
             attributes={k: ParseDict(v, Value()) for k, v in self._span.attributes.items()},
         )
 
+    @classmethod
+    def _from_otel_proto(cls, otel_proto_span) -> "Span":
+        """
+        Create a Span from an OpenTelemetry protobuf span.
+        This is an internal method used for receiving spans via OTel protocol.
+        """
+        trace_id = _otel_proto_bytes_to_id(otel_proto_span.trace_id)
+        span_id = _otel_proto_bytes_to_id(otel_proto_span.span_id)
+        parent_id = None
+        if otel_proto_span.parent_span_id:
+            parent_id = _otel_proto_bytes_to_id(otel_proto_span.parent_span_id)
 
-def _encode_span_id_to_byte(span_id: Optional[int]) -> bytes:
+        # Convert OTel proto status code directly to OTel SDK status
+        if otel_proto_span.status.code == OTelProtoStatus.STATUS_CODE_OK:
+            status_code = OTelStatusCode.OK
+        elif otel_proto_span.status.code == OTelProtoStatus.STATUS_CODE_ERROR:
+            status_code = OTelStatusCode.ERROR
+        else:
+            status_code = OTelStatusCode.UNSET
+
+        otel_span = OTelReadableSpan(
+            name=otel_proto_span.name,
+            context=build_otel_context(trace_id, span_id),
+            parent=build_otel_context(trace_id, parent_id) if parent_id else None,
+            start_time=otel_proto_span.start_time_unix_nano,
+            end_time=otel_proto_span.end_time_unix_nano,
+            attributes={
+                **{
+                    attr.key: _decode_otel_proto_anyvalue(attr.value)
+                    for attr in otel_proto_span.attributes
+                },
+                # Include the MLflow trace request ID
+                SpanAttributeKey.REQUEST_ID: generate_mlflow_trace_id_from_otel_trace_id(trace_id),
+            },
+            status=OTelStatus(status_code, otel_proto_span.status.message or None),
+            events=[
+                OTelEvent(
+                    name=event.name,
+                    timestamp=event.time_unix_nano,
+                    attributes={
+                        attr.key: _decode_otel_proto_anyvalue(attr.value)
+                        for attr in event.attributes
+                    },
+                )
+                for event in otel_proto_span.events
+            ],
+            resource=_OTelResource.get_empty(),
+        )
+
+        return cls(otel_span)
+
+    def _to_otel_proto(self) -> OTelProtoSpan:
+        """
+        Convert to OpenTelemetry protobuf span format for OTLP export.
+        This is an internal method used by the REST store for logging spans.
+
+        Returns:
+            An OpenTelemetry protobuf Span message.
+        """
+        otel_span = OTelProtoSpan()
+        otel_span.trace_id = bytes.fromhex(self._trace_id)
+        otel_span.span_id = bytes.fromhex(self.span_id)
+
+        otel_span.name = self.name
+        otel_span.start_time_unix_nano = self.start_time_ns
+        if self.end_time_ns:
+            otel_span.end_time_unix_nano = self.end_time_ns
+
+        if self.parent_id:
+            otel_span.parent_span_id = bytes.fromhex(self.parent_id)
+
+        otel_span.status.CopyFrom(self.status.to_otel_proto_status())
+
+        for key, value in self.attributes.items():
+            attr = otel_span.attributes.add()
+            attr.key = key
+            _set_otel_proto_anyvalue(attr.value, value)
+
+        for event in self.events:
+            otel_event = event._to_otel_proto()
+            otel_span.events.append(otel_event)
+
+        return otel_span
+
+
+def _encode_span_id_to_byte(span_id: int | None) -> bytes:
     # https://github.com/open-telemetry/opentelemetry-python/blob/e01fa0c77a7be0af77d008a888c2b6a707b05c3d/exporter/opentelemetry-exporter-otlp-proto-common/src/opentelemetry/exporter/otlp/proto/common/_internal/__init__.py#L131
     return span_id.to_bytes(length=8, byteorder="big", signed=False)
 
@@ -389,6 +485,12 @@ class LiveSpan(Span):
         self._attributes = _SpanAttributesRegistry(otel_span)
         self._attributes.set(SpanAttributeKey.REQUEST_ID, trace_id)
         self._attributes.set(SpanAttributeKey.SPAN_TYPE, span_type)
+        # Track the original span name for deduplication purposes during span logging.
+        # Why: When traces contain multiple spans with identical names (e.g., multiple "LLM"
+        # or "query" spans), it's difficult for users to distinguish between them in the UI
+        # and logs. As spans are logged, we incrementally add numeric suffixes (_1, _2, etc.) to
+        # make each span uniquely identifiable within its trace
+        self._original_name = otel_span.name
 
     def set_span_type(self, span_type: str):
         """Set the type of the span."""
@@ -421,7 +523,7 @@ class LiveSpan(Span):
         """Set a single attribute to the span."""
         self._attributes.set(key, value)
 
-    def set_status(self, status: Union[SpanStatusCode, str]):
+    def set_status(self, status: SpanStatusCode | str):
         """
         Set the status of the span.
 
@@ -455,7 +557,7 @@ class LiveSpan(Span):
         """
         self._span.add_event(event.name, event.attributes, event.timestamp)
 
-    def record_exception(self, exception: Union[str, Exception]):
+    def record_exception(self, exception: str | Exception):
         """
         Record an exception on the span, adding an exception event and setting span status to ERROR.
 
@@ -473,14 +575,19 @@ class LiveSpan(Span):
                 INVALID_PARAMETER_VALUE,
             )
 
-        self.set_status(SpanStatusCode.ERROR)
+        self.set_status(
+            SpanStatus(
+                status_code=SpanStatusCode.ERROR,
+                description=f"{type(exception).__name__}: {exception}",
+            )
+        )
 
     def end(
         self,
-        outputs: Optional[Any] = None,
-        attributes: Optional[dict[str, Any]] = None,
-        status: Optional[Union[SpanStatus, str]] = None,
-        end_time_ns: Optional[int] = None,
+        outputs: Any | None = None,
+        attributes: dict[str, Any] | None = None,
+        status: SpanStatus | str | None = None,
+        end_time_ns: int | None = None,
     ):
         """
         End the span.
@@ -513,6 +620,9 @@ class LiveSpan(Span):
             if self.status.status_code != SpanStatusCode.ERROR:
                 self.set_status(SpanStatus(SpanStatusCode.OK))
 
+            # Apply span processors
+            apply_span_processors(self)
+
             self._span.end(end_time=end_time_ns)
 
         except Exception as e:
@@ -538,9 +648,10 @@ class LiveSpan(Span):
     def from_immutable_span(
         cls,
         span: Span,
-        parent_span_id: Optional[str] = None,
-        trace_id: Optional[str] = None,
-        otel_trace_id: Optional[str] = None,
+        parent_span_id: str | None = None,
+        trace_id: str | None = None,
+        experiment_id: str | None = None,
+        otel_trace_id: str | None = None,
         end_trace: bool = True,
     ) -> "LiveSpan":
         """
@@ -558,6 +669,8 @@ class LiveSpan(Span):
                 If it is None, the span will be created as a root span.
             trace_id: The trace ID to be set on the new span. Specify this if you want to
                 create the new span with a particular trace ID.
+            experiment_id: The experiment ID to be set on the new span. If not specified, the
+                experiment ID will be set to the current experiment ID.
             otel_trace_id: The OpenTelemetry trace ID of the new span in hex encoded format.
                 If not specified, the newly generated trace ID will be used.
             end_trace: Whether to end the trace after cloning the span. Default is True.
@@ -577,6 +690,7 @@ class LiveSpan(Span):
             name=span.name,
             parent=parent_span._span if parent_span else None,
             start_time_ns=span.start_time_ns,
+            experiment_id=experiment_id,
         )
         # The latter one from attributes is the newly generated trace ID by the span processor.
         trace_id = trace_id or json.loads(otel_span.attributes.get(SpanAttributeKey.REQUEST_ID))
@@ -587,8 +701,10 @@ class LiveSpan(Span):
         clone_span.set_attributes(
             {k: v for k, v in span.attributes.items() if k != SpanAttributeKey.REQUEST_ID}
         )
-        clone_span.set_inputs(span.inputs)
-        clone_span.set_outputs(span.outputs)
+        if span.inputs:
+            clone_span.set_inputs(span.inputs)
+        if span.outputs:
+            clone_span.set_outputs(span.outputs)
         for event in span.events:
             clone_span.add_event(event)
 
@@ -693,15 +809,15 @@ class NoOpSpan(Span):
     def add_event(self, event: SpanEvent):
         pass
 
-    def record_exception(self, exception: Union[str, Exception]):
+    def record_exception(self, exception: str | Exception):
         pass
 
     def end(
         self,
-        outputs: Optional[Any] = None,
-        attributes: Optional[dict[str, Any]] = None,
-        status: Optional[Union[SpanStatus, str]] = None,
-        end_time_ns: Optional[int] = None,
+        outputs: Any | None = None,
+        attributes: dict[str, Any] | None = None,
+        status: SpanStatus | str | None = None,
+        end_time_ns: int | None = None,
     ):
         pass
 
@@ -728,11 +844,10 @@ class _SpanAttributesRegistry:
         if serialized_value:
             try:
                 return json.loads(serialized_value)
-            except Exception as e:
-                _logger.warning(
-                    f"Failed to get value for key {key}, make sure you set the attribute "
-                    f"on mlflow Span class instead of directly to the OpenTelemetry span. {e}"
-                )
+            except Exception:
+                # If failed to deserialize (e.g., string value is directly set to OTel span),
+                # return the original value as is.
+                return serialized_value
 
     def set(self, key: str, value: Any):
         if not isinstance(key, str):
