@@ -7,6 +7,14 @@ PROMPT_TEXT_TAG_KEY = "mlflow.prompt.text"
 
 LINKED_PROMPTS_TAG_KEY = "mlflow.linkedPrompts"
 
+# Unity Catalog tags cannot contain dots
+PROMPT_TYPE_TAG_KEY = "_mlflow_prompt_type"
+RESPONSE_FORMAT_TAG_KEY = "_mlflow_prompt_response_format"
+
+# Prompt types
+PROMPT_TYPE_TEXT = "text"
+PROMPT_TYPE_CHAT = "chat"
+
 # A special tag to store associated run IDs for prompts
 PROMPT_ASSOCIATED_RUN_IDS_TAG_KEY = "mlflow.prompt.associatedRunIds"
 
