@@ -7,9 +7,10 @@ import re
 import shutil
 from contextlib import contextmanager
 from dataclasses import dataclass
-from typing import Optional, Union
+from typing import Any
 
 import google.protobuf.empty_pb2
+from pydantic import BaseModel
 
 import mlflow
 from mlflow.entities import Run
@@ -17,6 +18,12 @@ from mlflow.entities.logged_model import LoggedModel
 from mlflow.entities.model_registry.prompt import Prompt
 from mlflow.entities.model_registry.prompt_version import PromptVersion
 from mlflow.exceptions import MlflowException, RestException
+from mlflow.prompt.constants import (
+    PROMPT_TYPE_CHAT,
+    PROMPT_TYPE_TAG_KEY,
+    PROMPT_TYPE_TEXT,
+    RESPONSE_FORMAT_TAG_KEY,
+)
 from mlflow.protos.databricks_pb2 import (
     INTERNAL_ERROR,
     INVALID_PARAMETER_VALUE,
@@ -186,7 +193,7 @@ class _CatalogSchemaFilter:
 
     catalog_name: str
     schema_name: str
-    remaining_filter: Optional[str]
+    remaining_filter: str | None
 
 
 def _require_arg_unspecified(arg_name, arg_value, default_values=None, message=None):
@@ -881,13 +888,13 @@ class UcModelRegistryStore(BaseRestStore):
                 if not os.path.exists(source) and not is_fuse_or_uc_volumes_uri(local_model_dir):
                     shutil.rmtree(local_model_dir)
 
-    def _get_logged_model_from_model_id(self, model_id) -> Optional[LoggedModel]:
+    def _get_logged_model_from_model_id(self, model_id) -> LoggedModel | None:
         # load the MLflow LoggedModel by model_id and
         if model_id is None:
             return None
         return mlflow.get_logged_model(model_id)
 
-    def create_model_version(
+    def _create_model_version_with_optional_signature_validation(
         self,
         name,
         source,
@@ -896,10 +903,17 @@ class UcModelRegistryStore(BaseRestStore):
         run_link=None,
         description=None,
         local_model_path=None,
-        model_id: Optional[str] = None,
+        model_id: str | None = None,
+        bypass_signature_validation: bool = False,
+        source_workspace_id: str | None = None,
     ):
         """
-        Create a new model version from given source and run ID.
+        Private method to create a new model version from given source and run ID, with optional
+        bypass of signature validation. This bypass is currently only used by the
+        DatabricksWorkspaceModelRegistryRestStore to migrate model versions from the Databricks
+        workspace registry to Unity Catalog via copy_model_version. We do not want to allow
+        normal use of create_model_version to bypass signature validation, so we have this
+        private method.
 
         Args:
             name: Registered model name.
@@ -916,6 +930,9 @@ class UcModelRegistryStore(BaseRestStore):
                 mlflow.<flavor>.log_model(..., registered_model_name) call.
             model_id: The ID of the model (from an Experiment) that is being promoted to a
                 registered model version, if applicable.
+            bypass_signature_validation: Whether to bypass signature validation.
+            source_workspace_id: The workspace ID of the source run. If not provided,
+                it will be fetched from the run headers.
 
         Returns:
             A single object of :py:class:`mlflow.entities.model_registry.ModelVersion`
@@ -926,7 +943,8 @@ class UcModelRegistryStore(BaseRestStore):
         if logged_model:
             run_id = logged_model.source_run_id
         headers, run = self._get_run_and_headers(run_id)
-        source_workspace_id = self._get_workspace_id(headers)
+        if source_workspace_id is None:
+            source_workspace_id = self._get_workspace_id(headers)
         notebook_id = self._get_notebook_id(run)
         lineage_securable_list = self._get_lineage_input_sources(run)
         job_id = self._get_job_id(run)
@@ -951,7 +969,8 @@ class UcModelRegistryStore(BaseRestStore):
             extra_headers = {_DATABRICKS_LINEAGE_ID_HEADER: header_base64}
         full_name = get_full_name_from_sc(name, self.spark)
         with self._local_model_dir(source, local_model_path) as local_model_dir:
-            self._validate_model_signature(local_model_dir)
+            if not bypass_signature_validation:
+                self._validate_model_signature(local_model_dir)
             self._download_model_weights_if_not_saved(local_model_dir)
             feature_deps = get_feature_dependencies(local_model_dir)
             other_model_deps = get_model_version_dependencies(local_model_dir)
@@ -979,6 +998,52 @@ class UcModelRegistryStore(BaseRestStore):
             )
             return model_version_from_uc_proto(finalized_mv)
 
+    def create_model_version(
+        self,
+        name,
+        source,
+        run_id=None,
+        tags=None,
+        run_link=None,
+        description=None,
+        local_model_path=None,
+        model_id: str | None = None,
+    ):
+        """
+        Create a new model version from given source and run ID.
+
+        Args:
+            name: Registered model name.
+            source: URI indicating the location of the model artifacts.
+            run_id: Run ID from MLflow tracking server that generated the model.
+            tags: A list of :py:class:`mlflow.entities.model_registry.ModelVersionTag`
+                instances associated with this model version.
+            run_link: Link to the run from an MLflow tracking server that generated this model.
+            description: Description of the version.
+            local_model_path: Local path to the MLflow model, if it's already accessible on the
+                local filesystem. Can be used by AbstractStores that upload model version files
+                to the model registry to avoid a redundant download from the source location when
+                logging and registering a model via a single
+                mlflow.<flavor>.log_model(..., registered_model_name) call.
+            model_id: The ID of the model (from an Experiment) that is being promoted to a
+                registered model version, if applicable.
+
+        Returns:
+            A single object of :py:class:`mlflow.entities.model_registry.ModelVersion`
+            created in the backend.
+        """
+        return self._create_model_version_with_optional_signature_validation(
+            name=name,
+            source=source,
+            run_id=run_id,
+            tags=tags,
+            run_link=run_link,
+            description=description,
+            local_model_path=local_model_path,
+            model_id=model_id,
+            bypass_signature_validation=False,
+        )
+
     def _get_artifact_repo(self, model_version, model_name=None):
         def base_credential_refresh_def():
             return self._get_temporary_model_version_write_credentials(
@@ -1219,8 +1284,8 @@ class UcModelRegistryStore(BaseRestStore):
     def create_prompt(
         self,
         name: str,
-        description: Optional[str] = None,
-        tags: Optional[dict[str, str]] = None,
+        description: str | None = None,
+        tags: dict[str, str] | None = None,
     ) -> Prompt:
         """
         Create a new prompt in Unity Catalog (metadata only, no initial version).
@@ -1244,10 +1309,10 @@ class UcModelRegistryStore(BaseRestStore):
 
     def search_prompts(
         self,
-        filter_string: Optional[str] = None,
-        max_results: Optional[int] = None,
-        order_by: Optional[list[str]] = None,
-        page_token: Optional[str] = None,
+        filter_string: str | None = None,
+        max_results: int | None = None,
+        order_by: list[str] | None = None,
+        page_token: str | None = None,
     ) -> PagedList[Prompt]:
         """
         Search for prompts in Unity Catalog.
@@ -1290,9 +1355,7 @@ class UcModelRegistryStore(BaseRestStore):
 
         return PagedList(prompts, response_proto.next_page_token)
 
-    def _parse_catalog_schema_from_filter(
-        self, filter_string: Optional[str]
-    ) -> _CatalogSchemaFilter:
+    def _parse_catalog_schema_from_filter(self, filter_string: str | None) -> _CatalogSchemaFilter:
         """
         Parse catalog and schema from filter string for Unity Catalog using regex.
 
@@ -1391,7 +1454,7 @@ class UcModelRegistryStore(BaseRestStore):
             proto_name=DeletePromptTagRequest,
         )
 
-    def get_prompt(self, name: str) -> Optional[Prompt]:
+    def get_prompt(self, name: str) -> Prompt | None:
         """
         Get prompt by name from Unity Catalog.
         """
@@ -1416,9 +1479,10 @@ class UcModelRegistryStore(BaseRestStore):
     def create_prompt_version(
         self,
         name: str,
-        template: str,
-        description: Optional[str] = None,
-        tags: Optional[dict[str, str]] = None,
+        template: str | list[dict[str, Any]],
+        description: str | None = None,
+        tags: dict[str, str] | None = None,
+        response_format: BaseModel | dict[str, Any] | None = None,
     ) -> PromptVersion:
         """
         Create a new prompt version in Unity Catalog.
@@ -1433,8 +1497,19 @@ class UcModelRegistryStore(BaseRestStore):
         # We don't set it here as it's generated server-side
         if description:
             prompt_version_proto.description = description
-        if tags:
-            prompt_version_proto.tags.extend(mlflow_tags_to_proto_version_tags(tags))
+
+        final_tags = tags.copy() if tags else {}
+        if response_format:
+            final_tags[RESPONSE_FORMAT_TAG_KEY] = json.dumps(
+                PromptVersion.convert_response_format_to_dict(response_format)
+            )
+        if isinstance(template, str):
+            final_tags[PROMPT_TYPE_TAG_KEY] = PROMPT_TYPE_TEXT
+        else:
+            final_tags[PROMPT_TYPE_TAG_KEY] = PROMPT_TYPE_CHAT
+
+        if final_tags:
+            prompt_version_proto.tags.extend(mlflow_tags_to_proto_version_tags(final_tags))
 
         req_body = message_to_json(
             CreatePromptVersionRequest(
@@ -1452,7 +1527,7 @@ class UcModelRegistryStore(BaseRestStore):
         )
         return proto_to_mlflow_prompt(response_proto)
 
-    def get_prompt_version(self, name: str, version: Union[str, int]) -> Optional[PromptVersion]:
+    def get_prompt_version(self, name: str, version: str | int) -> PromptVersion | None:
         """
         Get a specific prompt version from Unity Catalog.
         """
@@ -1477,7 +1552,7 @@ class UcModelRegistryStore(BaseRestStore):
                 return None
             raise
 
-    def delete_prompt_version(self, name: str, version: Union[str, int]) -> None:
+    def delete_prompt_version(self, name: str, version: str | int) -> None:
         """
         Delete a prompt version from Unity Catalog.
         """
@@ -1494,7 +1569,7 @@ class UcModelRegistryStore(BaseRestStore):
         )
 
     def search_prompt_versions(
-        self, name: str, max_results: Optional[int] = None, page_token: Optional[str] = None
+        self, name: str, max_results: int | None = None, page_token: str | None = None
     ) -> SearchPromptVersionsResponse:
         """
         Search prompt versions for a given prompt name in Unity Catalog.
@@ -1521,9 +1596,7 @@ class UcModelRegistryStore(BaseRestStore):
             proto_name=SearchPromptVersionsRequest,
         )
 
-    def set_prompt_version_tag(
-        self, name: str, version: Union[str, int], key: str, value: str
-    ) -> None:
+    def set_prompt_version_tag(self, name: str, version: str | int, key: str, value: str) -> None:
         """
         Set a tag on a prompt version in Unity Catalog.
         """
@@ -1541,7 +1614,7 @@ class UcModelRegistryStore(BaseRestStore):
             proto_name=SetPromptVersionTagRequest,
         )
 
-    def delete_prompt_version_tag(self, name: str, version: Union[str, int], key: str) -> None:
+    def delete_prompt_version_tag(self, name: str, version: str | int, key: str) -> None:
         """
         Delete a tag from a prompt version in Unity Catalog.
         """
@@ -1559,7 +1632,7 @@ class UcModelRegistryStore(BaseRestStore):
             proto_name=DeletePromptVersionTagRequest,
         )
 
-    def get_prompt_version_by_alias(self, name: str, alias: str) -> Optional[PromptVersion]:
+    def get_prompt_version_by_alias(self, name: str, alias: str) -> PromptVersion | None:
         """
         Get a prompt version by alias from Unity Catalog.
         """
@@ -1584,7 +1657,7 @@ class UcModelRegistryStore(BaseRestStore):
                 return None
             raise
 
-    def set_prompt_alias(self, name: str, alias: str, version: Union[str, int]) -> None:
+    def set_prompt_alias(self, name: str, alias: str, version: str | int) -> None:
         """
         Set an alias for a prompt version in Unity Catalog.
         """
