@@ -2,7 +2,6 @@ import functools
 import os
 import shutil
 from contextlib import contextmanager
-from typing import Optional
 
 import mlflow
 from mlflow.exceptions import MlflowException
@@ -39,7 +38,6 @@ from mlflow.utils._unity_catalog_utils import (
     get_artifact_repo_from_storage_info,
     get_full_name_from_sc,
 )
-from mlflow.utils.annotations import experimental
 from mlflow.utils.oss_registry_utils import get_oss_host_creds
 from mlflow.utils.proto_json_utils import message_to_json
 from mlflow.utils.rest_utils import (
@@ -71,7 +69,6 @@ def _require_arg_unspecified(arg_name, arg_value, default_values=None, message=N
         _raise_unsupported_arg(arg_name, message)
 
 
-@experimental
 class UnityCatalogOssStore(BaseRestStore):
     """
     Client for an Open Source Unity Catalog Server accessed via REST API calls.
@@ -268,7 +265,7 @@ class UnityCatalogOssStore(BaseRestStore):
         run_link=None,
         description=None,
         local_model_path=None,
-        model_id: Optional[str] = None,
+        model_id: str | None = None,
     ):
         with self._local_model_dir(source, local_model_path) as local_model_dir:
             [catalog_name, schema_name, model_name] = name.split(".")
@@ -433,7 +430,7 @@ class UnityCatalogOssStore(BaseRestStore):
             is_oss=True,
         )
 
-    def _get_temporary_model_version_write_credentials_oss(  # noqa: D417
+    def _get_temporary_model_version_write_credentials_oss(
         self, model_name, catalog_name, schema_name, version
     ):
         """
