@@ -1,7 +1,5 @@
 """allow nulls for metric values
 
-Revision ID: 181f10493468
-Revises: 90e64c465722
 Create Date: 2019-07-10 22:40:18.787993
 
 """
