@@ -1,7 +1,5 @@
 """Add Model Aliases table
 
-Revision ID: 3500859a5d39
-Revises: 97727af70f4d
 Create Date: 2023-03-09 15:33:54.951736
 
 """
