@@ -1,7 +1,7 @@
 import logging
-from typing import Optional
 
 from mlflow.entities.model_registry import ModelVersion, RegisteredModel
+from mlflow.entities.webhook import Webhook, WebhookEvent, WebhookStatus, WebhookTestResult
 from mlflow.protos.model_registry_pb2 import (
     CreateModelVersion,
     CreateRegisteredModel,
@@ -26,17 +26,28 @@ from mlflow.protos.model_registry_pb2 import (
     UpdateModelVersion,
     UpdateRegisteredModel,
 )
+from mlflow.protos.webhooks_pb2 import (
+    CreateWebhook,
+    DeleteWebhook,
+    GetWebhook,
+    ListWebhooks,
+    TestWebhook,
+    UpdateWebhook,
+    WebhookService,
+)
 from mlflow.store.entities.paged_list import PagedList
 from mlflow.store.model_registry.base_rest_store import BaseRestStore
 from mlflow.utils.proto_json_utils import message_to_json
 from mlflow.utils.rest_utils import (
     _REST_API_PATH_PREFIX,
+    call_endpoint,
     extract_all_api_info_for_service,
     extract_api_info_for_service,
 )
 
 _METHOD_TO_INFO = extract_api_info_for_service(ModelRegistryService, _REST_API_PATH_PREFIX)
 _METHOD_TO_ALL_INFO = extract_all_api_info_for_service(ModelRegistryService, _REST_API_PATH_PREFIX)
+_WEBHOOK_METHOD_TO_INFO = extract_api_info_for_service(WebhookService, _REST_API_PATH_PREFIX)
 
 _logger = logging.getLogger(__name__)
 
@@ -60,6 +71,21 @@ class RestStore(BaseRestStore):
     def _get_all_endpoints_from_method(self, method):
         return _METHOD_TO_ALL_INFO[method]
 
+    def _get_webhook_endpoint_from_method(self, method):
+        return _WEBHOOK_METHOD_TO_INFO[method]
+
+    def _call_webhook_endpoint(
+        self,
+        api,
+        json_body: str | None = None,
+        webhook_id: str | None = None,
+    ):
+        endpoint, method = self._get_webhook_endpoint_from_method(api)
+        if webhook_id:
+            endpoint = endpoint.format(webhook_id=webhook_id)
+        response_proto = self._get_response_from_method(api)
+        return call_endpoint(self.get_host_creds(), endpoint, method, json_body, response_proto)
+
     # CRUD API for RegisteredModel objects
 
     def create_registered_model(self, name, tags=None, description=None, deployment_job_id=None):
@@ -238,7 +264,7 @@ class RestStore(BaseRestStore):
         run_link=None,
         description=None,
         local_model_path=None,
-        model_id: Optional[str] = None,
+        model_id: str | None = None,
     ):
         """
         Create a new model version from given source and run ID.
@@ -479,3 +505,81 @@ class RestStore(BaseRestStore):
         req_body = message_to_json(GetModelVersionByAlias(name=name, alias=alias))
         response_proto = self._call_endpoint(GetModelVersionByAlias, req_body)
         return ModelVersion.from_proto(response_proto.model_version)
+
+    # Webhook APIs
+    def create_webhook(
+        self,
+        name: str,
+        url: str,
+        events: list[WebhookEvent],
+        description: str | None = None,
+        secret: str | None = None,
+        status: WebhookStatus | None = None,
+    ) -> Webhook:
+        req_body = message_to_json(
+            CreateWebhook(
+                name=name,
+                url=url,
+                events=[e.to_proto() for e in events],
+                description=description,
+                secret=secret,
+                status=status.to_proto() if status else None,
+            )
+        )
+        response_proto = self._call_webhook_endpoint(CreateWebhook, req_body)
+        return Webhook.from_proto(response_proto.webhook)
+
+    def get_webhook(self, webhook_id: str) -> Webhook:
+        response_proto = self._call_webhook_endpoint(GetWebhook, webhook_id=webhook_id)
+        return Webhook.from_proto(response_proto.webhook)
+
+    def list_webhooks(
+        self,
+        max_results: int | None = None,
+        page_token: str | None = None,
+    ) -> PagedList[Webhook]:
+        req_body = message_to_json(ListWebhooks(max_results=max_results, page_token=page_token))
+        response_proto = self._call_webhook_endpoint(ListWebhooks, req_body)
+        webhooks = [Webhook.from_proto(webhook) for webhook in response_proto.webhooks]
+        return PagedList(webhooks, response_proto.next_page_token)
+
+    def update_webhook(
+        self,
+        webhook_id: str,
+        name: str | None = None,
+        description: str | None = None,
+        url: str | None = None,
+        events: list[WebhookEvent] | None = None,
+        secret: str | None = None,
+        status: WebhookStatus | None = None,
+    ) -> Webhook:
+        req_body = message_to_json(
+            UpdateWebhook(
+                name=name,
+                description=description,
+                url=url,
+                events=[e.to_proto() for e in events] if events else None,
+                secret=secret,
+                status=status.to_proto() if status else None,
+            )
+        )
+        response_proto = self._call_webhook_endpoint(UpdateWebhook, req_body, webhook_id=webhook_id)
+        return Webhook.from_proto(response_proto.webhook)
+
+    def delete_webhook(self, webhook_id: str) -> None:
+        self._call_webhook_endpoint(DeleteWebhook, webhook_id=webhook_id)
+
+    def test_webhook(self, webhook_id: str, event: WebhookEvent | None = None) -> WebhookTestResult:
+        """
+        Test the webhook by sending a test event to the specified URL.
+
+        Args:
+            webhook_id: The ID of the webhook to test.
+            event: Optional event type to test. If not specified, uses the first event from webhook.
+
+        Returns:
+            WebhookTestResult indicating success/failure and response details
+        """
+        req_body = message_to_json(TestWebhook(event=event.to_proto() if event else None))
+        response_proto = self._call_webhook_endpoint(TestWebhook, req_body, webhook_id=webhook_id)
+        return WebhookTestResult.from_proto(response_proto.result)
