@@ -21,3 +21,5 @@ SEARCH_MAX_RESULTS_THRESHOLD = 50000
 GET_METRIC_HISTORY_MAX_RESULTS = 25000
 SEARCH_TRACES_DEFAULT_MAX_RESULTS = 100
 SEARCH_LOGGED_MODEL_MAX_RESULTS_DEFAULT = 100
+SEARCH_EVALUATION_DATASETS_MAX_RESULTS = 50
+LOAD_DATASET_RECORDS_MAX_RESULTS = 1000
