@@ -7,7 +7,7 @@ import inspect
 import json
 import logging
 from contextvars import ContextVar
-from typing import TYPE_CHECKING, Any, Callable, Generator, Literal, Optional, Union
+from typing import TYPE_CHECKING, Any, Callable, Generator, Literal
 
 from cachetools import TTLCache
 from opentelemetry import trace as trace_api
@@ -27,6 +27,7 @@ from mlflow.tracing.constant import (
     STREAM_CHUNK_EVENT_VALUE_KEY,
     SpanAttributeKey,
 )
+from mlflow.tracing.destination import TraceDestination
 from mlflow.tracing.provider import is_tracing_enabled, safe_set_span_in_context
 from mlflow.tracing.trace_manager import InMemoryTraceManager
 from mlflow.tracing.utils import (
@@ -36,12 +37,10 @@ from mlflow.tracing.utils import (
     encode_span_id,
     exclude_immutable_tags,
     get_otel_attribute,
-    set_chat_attributes_special_case,
 )
 from mlflow.tracing.utils.search import extract_span_inputs_outputs, traces_to_df
-from mlflow.tracing.utils.warning import request_id_backward_compatible
 from mlflow.utils import get_results_from_paginated_fn
-from mlflow.utils.annotations import experimental
+from mlflow.utils.annotations import deprecated_parameter
 
 _logger = logging.getLogger(__name__)
 
@@ -59,12 +58,13 @@ _EVAL_REQUEST_ID_TO_TRACE_ID = TTLCache(maxsize=10000, ttl=3600)
 
 
 def trace(
-    func: Optional[Callable] = None,
-    name: Optional[str] = None,
+    func: Callable[..., Any] | None = None,
+    name: str | None = None,
     span_type: str = SpanType.UNKNOWN,
-    attributes: Optional[dict[str, Any]] = None,
-    output_reducer: Optional[Callable] = None,
-) -> Callable:
+    attributes: dict[str, Any] | None = None,
+    output_reducer: Callable[[list[Any]], Any] | None = None,
+    trace_destination: TraceDestination | None = None,
+) -> Callable[..., Any]:
     """
     A decorator that creates a new span for the decorated function.
 
@@ -158,6 +158,11 @@ def trace(
         attributes: A dictionary of attributes to set on the span.
         output_reducer: A function that reduces the outputs of the generator function into a
             single value to be set as the span output.
+        trace_destination: The destination to log the trace to, such as MLflow Experiment. If
+            not provided, the destination will be an active MLflow experiment or an destination
+            set by the :py:func:`mlflow.tracing.set_destination` function. This parameter
+            should only be used for root span and setting this for non-root spans will be
+            ignored with a warning.
     """
 
     def decorator(fn):
@@ -176,13 +181,14 @@ def trace(
                 span_type,
                 attributes,
                 output_reducer,
+                trace_destination,
             )
         else:
             if output_reducer is not None:
                 raise MlflowException.invalid_parameter_value(
                     "The output_reducer argument is only supported for generator functions."
                 )
-            wrapped = _wrap_function(original_fn, name, span_type, attributes)
+            wrapped = _wrap_function(original_fn, name, span_type, attributes, trace_destination)
 
         # If the original was a descriptor, wrap the result back as the same type of descriptor
         if is_classmethod:
@@ -196,11 +202,12 @@ def trace(
 
 
 def _wrap_function(
-    fn: Callable,
-    name: Optional[str] = None,
+    fn: Callable[..., Any],
+    name: str | None = None,
     span_type: str = SpanType.UNKNOWN,
-    attributes: Optional[dict[str, Any]] = None,
-) -> Callable:
+    attributes: dict[str, Any] | None = None,
+    trace_destination: TraceDestination | None = None,
+) -> Callable[..., Any]:
     class _WrappingContext:
         # define the wrapping logic as a coroutine to avoid code duplication
         # between sync and async cases
@@ -208,15 +215,17 @@ def _wrap_function(
         def _wrapping_logic(fn, args, kwargs):
             span_name = name or fn.__name__
 
-            with start_span(name=span_name, span_type=span_type, attributes=attributes) as span:
+            with start_span(
+                name=span_name,
+                span_type=span_type,
+                attributes=attributes,
+                trace_destination=trace_destination,
+            ) as span:
                 span.set_attribute(SpanAttributeKey.FUNCTION_NAME, fn.__name__)
                 inputs = capture_function_input_args(fn, args, kwargs)
                 span.set_inputs(inputs)
                 result = yield  # sync/async function output to be sent here
                 span.set_outputs(result)
-
-                set_chat_attributes_special_case(span, inputs=inputs, outputs=result)
-
                 try:
                     yield result
                 except GeneratorExit:
@@ -254,12 +263,13 @@ def _wrap_function(
 
 
 def _wrap_generator(
-    fn: Callable,
-    name: Optional[str] = None,
+    fn: Callable[..., Any],
+    name: str | None = None,
     span_type: str = SpanType.UNKNOWN,
-    attributes: Optional[dict[str, Any]] = None,
-    output_reducer: Optional[Callable] = None,
-) -> Callable:
+    attributes: dict[str, Any] | None = None,
+    output_reducer: Callable[[list[Any]], Any] | None = None,
+    trace_destination: TraceDestination | None = None,
+) -> Callable[..., Any]:
     """
     Wrap a generator function to create a span.
     Generator functions need special handling because of its lazy evaluation nature.
@@ -295,6 +305,7 @@ def _wrap_generator(
                 span_type=span_type,
                 attributes=attributes,
                 inputs=inputs,
+                experiment_id=getattr(trace_destination, "experiment_id", None),
             )
         except Exception as e:
             _logger.debug(f"Failed to start stream span: {e}")
@@ -302,10 +313,10 @@ def _wrap_generator(
 
     def _end_stream_span(
         span: LiveSpan,
-        inputs: Optional[dict[str, Any]] = None,
-        outputs: Optional[list[Any]] = None,
-        output_reducer: Optional[Callable] = None,
-        error: Optional[Exception] = None,
+        inputs: dict[str, Any] | None = None,
+        outputs: list[Any] | None = None,
+        output_reducer: Callable[[list[Any]], Any] | None = None,
+        error: Exception | None = None,
     ):
         if error:
             span.add_event(SpanEvent.from_exception(error))
@@ -318,7 +329,6 @@ def _wrap_generator(
             except Exception as e:
                 _logger.debug(f"Failed to reduce outputs from stream: {e}")
 
-        set_chat_attributes_special_case(span, inputs=inputs, outputs=outputs)
         span.end(outputs=outputs)
 
     def _record_chunk_event(span: LiveSpan, chunk: Any, chunk_index: int):
@@ -385,21 +395,24 @@ def _wrap_generator(
     return _wrap_function_safe(fn, wrapper)
 
 
-def _wrap_function_safe(fn: Callable, wrapper: Callable) -> Callable:
+def _wrap_function_safe(fn: Callable[..., Any], wrapper: Callable[..., Any]) -> Callable[..., Any]:
     wrapped = functools.wraps(fn)(wrapper)
     # Update the signature of the wrapper to match the signature of the original (safely)
     try:
         wrapped.__signature__ = inspect.signature(fn)
     except Exception:
         pass
+    # Add unique marker for MLflow trace detection
+    wrapped.__mlflow_traced__ = True
     return wrapped
 
 
 @contextlib.contextmanager
 def start_span(
     name: str = "span",
-    span_type: Optional[str] = SpanType.UNKNOWN,
-    attributes: Optional[dict[str, Any]] = None,
+    span_type: str | None = SpanType.UNKNOWN,
+    attributes: dict[str, Any] | None = None,
+    trace_destination: TraceDestination | None = None,
 ) -> Generator[LiveSpan, None, None]:
     """
     Context manager to create a new span and start it as the current span in the context.
@@ -451,12 +464,19 @@ def start_span(
         span_type: The type of the span. Can be either a string or
             a :py:class:`SpanType <mlflow.entities.SpanType>` enum value
         attributes: A dictionary of attributes to set on the span.
+        trace_destination: The destination to log the trace to, such as MLflow Experiment. If
+            not provided, the destination will be an active MLflow experiment or an destination
+            set by the :py:func:`mlflow.tracing.set_destination` function. This parameter
+            should only be used for root span and setting this for non-root spans will be
+            ignored with a warning.
 
     Returns:
         Yields an :py:class:`mlflow.entities.Span` that represents the created span.
     """
     try:
-        otel_span = provider.start_span_in_context(name)
+        otel_span = provider.start_span_in_context(
+            name, experiment_id=trace_destination.experiment_id if trace_destination else None
+        )
 
         # Create a new MLflow span and register it to the in-memory trace manager
         request_id = get_otel_attribute(otel_span, SpanAttributeKey.REQUEST_ID)
@@ -486,12 +506,12 @@ def start_span(
 def start_span_no_context(
     name: str,
     span_type: str = SpanType.UNKNOWN,
-    parent_span: Optional[LiveSpan] = None,
-    inputs: Optional[Any] = None,
-    attributes: Optional[dict[str, str]] = None,
-    tags: Optional[dict[str, str]] = None,
-    experiment_id: Optional[str] = None,
-    start_time_ns: Optional[int] = None,
+    parent_span: LiveSpan | None = None,
+    inputs: Any | None = None,
+    attributes: dict[str, str] | None = None,
+    tags: dict[str, str] | None = None,
+    experiment_id: str | None = None,
+    start_time_ns: int | None = None,
 ) -> LiveSpan:
     """
     Start a span without attaching it to the global tracing context.
@@ -586,8 +606,8 @@ def start_span_no_context(
     return NoOpSpan()
 
 
-@request_id_backward_compatible
-def get_trace(trace_id: str) -> Optional[Trace]:
+@deprecated_parameter("request_id", "trace_id")
+def get_trace(trace_id: str, silent: bool = False) -> Trace | None:
     """
     Get a trace by the given request ID if it exists.
 
@@ -597,7 +617,8 @@ def get_trace(trace_id: str) -> Optional[Trace]:
 
     Args:
         trace_id: The ID of the trace.
-
+        silent: If True, suppress the warning message when the trace is not found. The API will
+            return None without any warning. Default to False.
 
     .. code-block:: python
         :test:
@@ -621,26 +642,27 @@ def get_trace(trace_id: str) -> Optional[Trace]:
     try:
         return TracingClient().get_trace(trace_id)
     except MlflowException as e:
-        _logger.warning(
-            f"Failed to get trace from the tracking store: {e}"
-            "For full traceback, set logging level to debug.",
-            exc_info=_logger.isEnabledFor(logging.DEBUG),
-        )
+        if not silent:
+            _logger.warning(
+                f"Failed to get trace from the tracking store: {e}"
+                "For full traceback, set logging level to debug.",
+                exc_info=_logger.isEnabledFor(logging.DEBUG),
+            )
         return None
 
 
 def search_traces(
-    experiment_ids: Optional[list[str]] = None,
-    filter_string: Optional[str] = None,
-    max_results: Optional[int] = None,
-    order_by: Optional[list[str]] = None,
-    extract_fields: Optional[list[str]] = None,
-    run_id: Optional[str] = None,
-    return_type: Optional[Literal["pandas", "list"]] = None,
-    model_id: Optional[str] = None,
-    sql_warehouse_id: Optional[str] = None,
+    experiment_ids: list[str] | None = None,
+    filter_string: str | None = None,
+    max_results: int | None = None,
+    order_by: list[str] | None = None,
+    extract_fields: list[str] | None = None,
+    run_id: str | None = None,
+    return_type: Literal["pandas", "list"] | None = None,
+    model_id: str | None = None,
+    sql_warehouse_id: str | None = None,
     include_spans: bool = True,
-) -> Union["pandas.DataFrame", list[Trace]]:
+) -> "pandas.DataFrame" | list[Trace]:
     """
     Return traces that match the given list of search expressions within the experiments.
 
@@ -828,7 +850,7 @@ def search_traces(
     return results
 
 
-def get_current_active_span() -> Optional[LiveSpan]:
+def get_current_active_span() -> LiveSpan | None:
     """
     Get the current active span in the global context.
 
@@ -868,7 +890,7 @@ def get_current_active_span() -> Optional[LiveSpan]:
     return trace_manager.get_span_from_id(request_id, encode_span_id(otel_span.context.span_id))
 
 
-def get_active_trace_id() -> Optional[str]:
+def get_active_trace_id() -> str | None:
     """
     Get the active trace ID in the current process.
 
@@ -899,7 +921,7 @@ def get_active_trace_id() -> Optional[str]:
     return None
 
 
-def get_last_active_trace_id(thread_local: bool = False) -> Optional[str]:
+def get_last_active_trace_id(thread_local: bool = False) -> str | None:
     """
     Get the **LAST** active trace in the same process if exists.
 
@@ -951,12 +973,12 @@ def _set_last_active_trace_id(trace_id: str):
 
 
 def update_current_trace(
-    tags: Optional[dict[str, str]] = None,
-    metadata: Optional[dict[str, str]] = None,
-    client_request_id: Optional[str] = None,
-    request_preview: Optional[str] = None,
-    response_preview: Optional[str] = None,
-    state: Optional[Union[TraceState, str]] = None,
+    tags: dict[str, str] | None = None,
+    metadata: dict[str, str] | None = None,
+    client_request_id: str | None = None,
+    request_preview: str | None = None,
+    response_preview: str | None = None,
+    state: TraceState | str | None = None,
 ):
     """
     Update the current active trace with the given options.
@@ -1106,7 +1128,7 @@ def update_current_trace(
             if state not in (TraceState.OK, TraceState.ERROR):
                 raise _invalid_state_error(state)
 
-            trace.info.state = state
+            trace.info.state = TraceState(state) if isinstance(state, str) else state
 
         trace.info.tags.update(tags or {})
         trace.info.trace_metadata.update(metadata or {})
@@ -1114,7 +1136,7 @@ def update_current_trace(
             trace.info.client_request_id = str(client_request_id)
 
 
-@request_id_backward_compatible
+@deprecated_parameter("request_id", "trace_id")
 def set_trace_tag(trace_id: str, key: str, value: str):
     """
     Set a tag on the trace with the given trace ID.
@@ -1141,7 +1163,7 @@ def set_trace_tag(trace_id: str, key: str, value: str):
     TracingClient().set_trace_tag(trace_id, key, value)
 
 
-@request_id_backward_compatible
+@deprecated_parameter("request_id", "trace_id", version="3.0.0")
 def delete_trace_tag(trace_id: str, key: str) -> None:
     """
     Delete a tag on the trace with the given trace ID.
@@ -1167,8 +1189,7 @@ def delete_trace_tag(trace_id: str, key: str) -> None:
     TracingClient().delete_trace_tag(trace_id, key)
 
 
-@experimental
-def add_trace(trace: Union[Trace, dict[str, Any]], target: Optional[LiveSpan] = None):
+def add_trace(trace: Trace | dict[str, Any], target: LiveSpan | None = None):
     """
     Add a completed trace object into another trace.
 
@@ -1284,16 +1305,15 @@ def add_trace(trace: Union[Trace, dict[str, Any]], target: Optional[LiveSpan] =
         )
 
 
-@experimental
 def log_trace(
     name: str = "Task",
-    request: Optional[Any] = None,
-    response: Optional[Any] = None,
-    intermediate_outputs: Optional[dict[str, Any]] = None,
-    attributes: Optional[dict[str, Any]] = None,
-    tags: Optional[dict[str, str]] = None,
-    start_time_ms: Optional[int] = None,
-    execution_time_ms: Optional[int] = None,
+    request: Any | None = None,
+    response: Any | None = None,
+    intermediate_outputs: dict[str, Any] | None = None,
+    attributes: dict[str, Any] | None = None,
+    tags: dict[str, str] | None = None,
+    start_time_ms: int | None = None,
+    execution_time_ms: int | None = None,
 ) -> str:
     """
     Create a trace with a single root span.
