@@ -9,14 +9,16 @@ from collections import Counter
 from contextlib import contextmanager
 from dataclasses import asdict, is_dataclass
 from functools import lru_cache
-from typing import TYPE_CHECKING, Any, Optional, Union
+from typing import TYPE_CHECKING, Any, Generator
 
 from opentelemetry import trace as trace_api
+from opentelemetry.sdk.trace import ReadableSpan as OTelReadableSpan
 from opentelemetry.sdk.trace import Span as OTelSpan
 from packaging.version import Version
 
 from mlflow.exceptions import BAD_REQUEST, MlflowTracingException
 from mlflow.tracing.constant import (
+    ASSESSMENT_ID_PREFIX,
     TRACE_REQUEST_ID_PREFIX,
     SpanAttributeKey,
     TokenUsageKey,
@@ -33,10 +35,10 @@ SPANS_COLUMN_NAME = "spans"
 if TYPE_CHECKING:
     from mlflow.entities import LiveSpan, Trace
     from mlflow.pyfunc.context import Context
-    from mlflow.types.chat import ChatMessage, ChatTool
+    from mlflow.types.chat import ChatTool
 
 
-def capture_function_input_args(func, args, kwargs) -> Optional[dict[str, Any]]:
+def capture_function_input_args(func, args, kwargs) -> dict[str, Any] | None:
     try:
         func_signature = inspect.signature(func)
         bound_arguments = func_signature.bind(*args, **kwargs)
@@ -160,6 +162,17 @@ def decode_id(span_or_trace_id: str) -> int:
     return int(span_or_trace_id, 16)
 
 
+def get_mlflow_span_for_otel_span(span: OTelSpan) -> LiveSpan | None:
+    """
+    Get the active MLflow span for the given OpenTelemetry span.
+    """
+    from mlflow.tracing.trace_manager import InMemoryTraceManager
+
+    trace_id = get_otel_attribute(span, SpanAttributeKey.REQUEST_ID)
+    mlflow_span_id = encode_span_id(span.get_span_context().span_id)
+    return InMemoryTraceManager.get_instance().get_span_from_id(trace_id, mlflow_span_id)
+
+
 def build_otel_context(trace_id: int, span_id: int) -> trace_api.SpanContext:
     """
     Build an OpenTelemetry SpanContext object from the given trace and span IDs.
@@ -188,19 +201,22 @@ def deduplicate_span_names_in_place(spans: list[LiveSpan]):
     Args:
         spans: A list of spans to deduplicate.
     """
-    span_name_counter = Counter(span.name for span in spans)
+    # Use _original_name to handle incremental deduplication correctly
+    span_name_counter = Counter(span._original_name for span in spans)
     # Apply renaming only for duplicated spans
     span_name_counter = {name: 1 for name, count in span_name_counter.items() if count > 1}
     # Add index to the duplicated span names
     for span in spans:
-        if count := span_name_counter.get(span.name):
-            span_name_counter[span.name] += 1
-            span._span._name = f"{span.name}_{count}"
+        if count := span_name_counter.get(span._original_name):
+            span_name_counter[span._original_name] += 1
+            span._span._name = f"{span._original_name}_{count}"
 
 
-def aggregate_usage_from_spans(spans: list[LiveSpan]) -> Optional[dict[str, int]]:
+def aggregate_usage_from_spans(spans: list[LiveSpan]) -> dict[str, int] | None:
     """Aggregate token usage information from all spans in the trace."""
-    input_tokens, output_tokens, total_tokens = 0, 0, 0
+    input_tokens = 0
+    output_tokens = 0
+    total_tokens = 0
     has_usage_data = False
 
     span_id_to_spans = {span.span_id: span for span in spans}
@@ -233,7 +249,7 @@ def aggregate_usage_from_spans(spans: list[LiveSpan]) -> Optional[dict[str, int]
     }
 
 
-def get_otel_attribute(span: trace_api.Span, key: str) -> Optional[str]:
+def get_otel_attribute(span: trace_api.Span, key: str) -> str | None:
     """
     Get the attribute value from the OpenTelemetry span in a decoded format.
 
@@ -256,13 +272,13 @@ def _try_get_prediction_context():
     #     relies on numpy, which is not installed in skinny.
     try:
         from mlflow.pyfunc.context import get_prediction_context
-    except ImportError:
+    except (ImportError, KeyError):
         return
 
     return get_prediction_context()
 
 
-def maybe_get_request_id(is_evaluate=False) -> Optional[str]:
+def maybe_get_request_id(is_evaluate=False) -> str | None:
     """Get the request ID if the current prediction is as a part of MLflow model evaluation."""
     context = _try_get_prediction_context()
     if not context or (is_evaluate and not context.is_evaluate):
@@ -279,13 +295,13 @@ def maybe_get_request_id(is_evaluate=False) -> Optional[str]:
     return context.request_id
 
 
-def maybe_get_dependencies_schemas() -> Optional[dict]:
+def maybe_get_dependencies_schemas() -> dict[str, Any] | None:
     context = _try_get_prediction_context()
     if context:
         return context.dependencies_schemas
 
 
-def maybe_get_logged_model_id() -> Optional[str]:
+def maybe_get_logged_model_id() -> str | None:
     """
     Get the logged model ID associated with the current prediction context.
     """
@@ -298,13 +314,26 @@ def exclude_immutable_tags(tags: dict[str, str]) -> dict[str, str]:
     return {k: v for k, v in tags.items() if k not in IMMUTABLE_TAGS}
 
 
+def generate_mlflow_trace_id_from_otel_trace_id(otel_trace_id: int) -> str:
+    """
+    Generate an MLflow trace ID from an OpenTelemetry trace ID.
+
+    Args:
+        otel_trace_id: The OpenTelemetry trace ID as an integer.
+
+    Returns:
+        The MLflow trace ID string in format "tr-<hex_trace_id>".
+    """
+    return TRACE_REQUEST_ID_PREFIX + encode_trace_id(otel_trace_id)
+
+
 def generate_trace_id_v3(span: OTelSpan) -> str:
     """
     Generate a trace ID for the given span (V3 trace schema).
 
     The format will be "tr-<trace_id>" where the trace_id is hex-encoded Otel trace ID.
     """
-    return TRACE_REQUEST_ID_PREFIX + encode_trace_id(span.context.trace_id)
+    return generate_mlflow_trace_id_from_otel_trace_id(span.context.trace_id)
 
 
 def generate_request_id_v2() -> str:
@@ -334,7 +363,7 @@ def construct_full_inputs(func, *args, **kwargs) -> dict[str, Any]:
 
 
 @contextmanager
-def maybe_set_prediction_context(context: Optional["Context"]):
+def maybe_set_prediction_context(context: "Context" | None):
     """
     Set the prediction context if the given context
     is not None. Otherwise no-op.
@@ -348,66 +377,6 @@ def maybe_set_prediction_context(context: Optional["Context"]):
         yield
 
 
-def set_span_chat_messages(
-    span: LiveSpan,
-    messages: list[Union[dict, ChatMessage]],
-    append=False,
-):
-    """
-    Set the `mlflow.chat.messages` attribute on the specified span. This
-    attribute is used in the UI, and also by downstream applications that
-    consume trace data, such as MLflow evaluate.
-
-    Args:
-        span: The LiveSpan to add the attribute to
-        messages: A list of standardized chat messages (refer to the
-                 `spec <../llms/tracing/tracing-schema.html#chat-completion-spans>`_
-                 for details)
-        append: If True, the messages will be appended to the existing messages. Otherwise,
-                the attribute will be overwritten entirely. Default is False.
-                This is useful when you want to record messages incrementally, e.g., log
-                input messages first, and then log output messages later.
-
-    Example:
-
-    .. code-block:: python
-        :test:
-
-        import mlflow
-        from mlflow.tracing import set_span_chat_messages
-
-
-        @mlflow.trace
-        def f():
-            messages = [{"role": "user", "content": "hello"}]
-            span = mlflow.get_current_active_span()
-            set_span_chat_messages(span, messages)
-            return 0
-
-
-        f()
-    """
-    from mlflow.types.chat import ChatMessage
-
-    sanitized_messages = []
-    for message in messages:
-        if isinstance(message, dict):
-            ChatMessage.validate_compat(message)
-            sanitized_messages.append(message)
-        elif isinstance(message, ChatMessage):
-            # NB: ChatMessage is used for both request and response messages. In OpenAI's API spec,
-            #   some fields are only present in either the request or response (e.g., tool_call_id).
-            #   Those fields should not be recorded unless set explicitly, so we set
-            #   exclude_unset=True here to avoid recording unset fields.
-            sanitized_messages.append(message.model_dump_compat(exclude_unset=True))
-
-    if append:
-        existing_messages = span.get_attribute(SpanAttributeKey.CHAT_MESSAGES) or []
-        sanitized_messages = existing_messages + sanitized_messages
-
-    span.set_attribute(SpanAttributeKey.CHAT_MESSAGES, sanitized_messages)
-
-
 def set_span_chat_tools(span: LiveSpan, tools: list[ChatTool]):
     """
     Set the `mlflow.chat.tools` attribute on the specified span. This
@@ -475,42 +444,6 @@ def set_span_chat_tools(span: LiveSpan, tools: list[ChatTool]):
     span.set_attribute(SpanAttributeKey.CHAT_TOOLS, sanitized_tools)
 
 
-def set_chat_attributes_special_case(span: LiveSpan, inputs: Any, outputs: Any):
-    """
-    Set the `mlflow.chat.messages` and `mlflow.chat.tools` attributes on the specified span
-    based on the inputs and outputs of the function.
-
-    Usually those attributes are set by autologging integrations. This utility function handles
-    special cases where we want to set chat attributes for manually created spans via @mlflow.trace
-    decorator, such as ResponsesAgent tracing spans.
-    """
-    try:
-        from mlflow.openai.utils.chat_schema import set_span_chat_attributes
-        from mlflow.types.responses import ResponsesAgentResponse, ResponsesAgentStreamEvent
-
-        if isinstance(outputs, ResponsesAgentResponse):
-            inputs = inputs["request"].model_dump_compat()
-            set_span_chat_attributes(span, inputs, outputs)
-        elif isinstance(outputs, list) and all(
-            isinstance(o, ResponsesAgentStreamEvent) for o in outputs
-        ):
-            inputs = inputs["request"].model_dump_compat()
-            output_items = []
-            custom_outputs = None
-            for o in outputs:
-                if o.type == "response.output_item.done":
-                    output_items.append(o.item)
-                if o.custom_outputs:
-                    custom_outputs = o.custom_outputs
-            output = ResponsesAgentResponse(
-                output=output_items,
-                custom_outputs=custom_outputs,
-            )
-            set_span_chat_attributes(span, inputs, output)
-    except Exception:
-        pass
-
-
 def _calculate_percentile(sorted_data: list[float], percentile: float) -> float:
     """
     Calculate the percentile value from sorted data.
@@ -612,3 +545,64 @@ def update_trace_state_from_span_conditionally(trace, root_span):
     # and we should preserve it
     if trace.info.state == TraceState.IN_PROGRESS:
         trace.info.state = TraceState.from_otel_status(root_span.status)
+
+
+def get_experiment_id_for_trace(span: OTelReadableSpan) -> str:
+    """
+    Determine the experiment ID to associate with the trace.
+
+    The experiment ID can be configured in multiple ways, in order of precedence:
+      1. An experiment ID specified via the span creation API i.e. MlflowClient().start_trace()
+      2. An experiment ID specified via `mlflow.tracing.set_destination`
+      3. An experiment ID of an active run.
+      4. The default experiment ID
+
+    Args:
+        span: The OpenTelemetry ReadableSpan to extract experiment ID from.
+
+    Returns:
+        The experiment ID string to use for the trace.
+    """
+    from mlflow.tracing.provider import _MLFLOW_TRACE_USER_DESTINATION
+    from mlflow.tracking.fluent import _get_experiment_id, _get_latest_active_run
+
+    if experiment_id := get_otel_attribute(span, SpanAttributeKey.EXPERIMENT_ID):
+        return experiment_id
+
+    if destination := _MLFLOW_TRACE_USER_DESTINATION.get():
+        if exp_id := getattr(destination, "experiment_id", None):
+            return exp_id
+
+    if run := _get_latest_active_run():
+        return run.info.experiment_id
+
+    return _get_experiment_id()
+
+
+def generate_assessment_id() -> str:
+    """
+    Generates an assessment ID of the form 'a-<uuid4>' in hex string format.
+
+    Returns:
+        A unique identifier for an assessment that will be logged to a trace tag.
+    """
+    id = uuid.uuid4().hex
+    return f"{ASSESSMENT_ID_PREFIX}{id}"
+
+
+@contextmanager
+def _bypass_attribute_guard(span: OTelSpan) -> Generator[None, None, None]:
+    """
+    OpenTelemetry does not allow setting attributes if the span has end time defined.
+    https://github.com/open-telemetry/opentelemetry-python/blob/d327927d0274a320466feec6fba6d6ddb287dc5a/opentelemetry-sdk/src/opentelemetry/sdk/trace/__init__.py#L849-L851
+
+    However, we need to set some attributes within `on_end` handler of the span processor,
+    where the span is already marked as ended. This context manager is a hacky workaround
+    to bypass the attribute guard.
+    """
+    original_end_time = span._end_time
+    span._end_time = None
+    try:
+        yield
+    finally:
+        span._end_time = original_end_time
