@@ -8,7 +8,7 @@ import logging
 import os
 import sys
 from itertools import zip_longest
-from typing import Any, Literal, Optional
+from typing import TYPE_CHECKING, Any, Literal
 
 from mlflow.entities import (
     ExperimentTag,
@@ -25,23 +25,43 @@ from mlflow.entities import (
     RunTag,
     ViewType,
 )
+
+if TYPE_CHECKING:
+    from mlflow.entities import EvaluationDataset
 from mlflow.entities.dataset_input import DatasetInput
 from mlflow.environment_variables import MLFLOW_SUPPRESS_PRINTING_URL_TO_STDOUT
 from mlflow.exceptions import MlflowException
-from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE, ErrorCode
+from mlflow.protos.databricks_pb2 import (
+    INVALID_PARAMETER_VALUE,
+    ErrorCode,
+)
 from mlflow.store.artifact.artifact_repo import ArtifactRepository
 from mlflow.store.artifact.artifact_repository_registry import get_artifact_repository
+from mlflow.store.entities.paged_list import PagedList
 from mlflow.store.tracking import (
     GET_METRIC_HISTORY_MAX_RESULTS,
     SEARCH_MAX_RESULTS_DEFAULT,
 )
 from mlflow.store.tracking.rest_store import RestStore
+from mlflow.telemetry.events import (
+    CreateDatasetEvent,
+    CreateExperimentEvent,
+    CreateLoggedModelEvent,
+    CreateRunEvent,
+    GetLoggedModelEvent,
+    LogBatchEvent,
+    LogDatasetEvent,
+    LogMetricEvent,
+    LogParamEvent,
+)
+from mlflow.telemetry.track import record_usage_event
 from mlflow.tracking._tracking_service import utils
+from mlflow.tracking.context import registry as context_registry
 from mlflow.tracking.metric_value_conversion_utils import convert_metric_value_to_float_if_possible
 from mlflow.utils import chunk_list
 from mlflow.utils.async_logging.run_operations import RunOperations, get_combined_run_operations
 from mlflow.utils.databricks_utils import get_workspace_url, is_in_databricks_notebook
-from mlflow.utils.mlflow_tags import MLFLOW_USER
+from mlflow.utils.mlflow_tags import MLFLOW_RUN_IS_EVALUATION, MLFLOW_USER
 from mlflow.utils.string_utils import is_string_type
 from mlflow.utils.time import get_current_time_millis
 from mlflow.utils.uri import add_databricks_profile_info_to_artifact_uri, is_databricks_uri
@@ -132,6 +152,7 @@ class TrackingServiceClient:
             token = paged_history.token
         return history
 
+    @record_usage_event(CreateRunEvent)
     def create_run(self, experiment_id, start_time=None, tags=None, run_name=None):
         """Create a :py:class:`mlflow.entities.Run` object that can be associated with
         metrics, parameters, artifacts, etc.
@@ -257,6 +278,7 @@ class TrackingServiceClient:
         """
         return self.store.get_experiment_by_name(name)
 
+    @record_usage_event(CreateExperimentEvent)
     def create_experiment(self, name, artifact_location=None, tags=None):
         """Create an experiment.
 
@@ -306,6 +328,7 @@ class TrackingServiceClient:
         """
         self.store.rename_experiment(experiment_id, new_name)
 
+    @record_usage_event(LogMetricEvent)
     def log_metric(
         self,
         run_id,
@@ -314,10 +337,10 @@ class TrackingServiceClient:
         timestamp=None,
         step=None,
         synchronous=True,
-        dataset_name: Optional[str] = None,
-        dataset_digest: Optional[str] = None,
-        model_id: Optional[str] = None,
-    ) -> Optional[RunOperations]:
+        dataset_name: str | None = None,
+        dataset_digest: str | None = None,
+        model_id: str | None = None,
+    ) -> RunOperations | None:
         """Log a metric against the run ID.
 
         Args:
@@ -362,6 +385,7 @@ class TrackingServiceClient:
         else:
             return self.store.log_metric_async(run_id, metric)
 
+    @record_usage_event(LogParamEvent)
     def log_param(self, run_id, key, value, synchronous=True):
         """Log a parameter (e.g. model hyperparameter) against the run ID. Value is converted to
         a string.
@@ -405,7 +429,16 @@ class TrackingServiceClient:
         tag = ExperimentTag(key, str(value))
         self.store.set_experiment_tag(experiment_id, tag)
 
-    def set_tag(self, run_id, key, value, synchronous=True) -> Optional[RunOperations]:
+    def delete_experiment_tag(self, experiment_id, key):
+        """Delete a tag from the experiment with the specified ID.
+
+        Args:
+            experiment_id: String ID of the experiment.
+            key: Name of the tag to be deleted.
+        """
+        self.store.delete_experiment_tag(experiment_id, key)
+
+    def set_tag(self, run_id, key, value, synchronous=True) -> RunOperations | None:
         """Set a tag on the run with the specified ID. Value is converted to a string.
 
         Args:
@@ -467,9 +500,10 @@ class TrackingServiceClient:
             run_name=name,
         )
 
+    @record_usage_event(LogBatchEvent)
     def log_batch(
         self, run_id, metrics=(), params=(), tags=(), synchronous=True
-    ) -> Optional[RunOperations]:
+    ) -> RunOperations | None:
         """Log multiple metrics, params, and/or tags.
 
         Args:
@@ -555,11 +589,12 @@ class TrackingServiceClient:
             # Merge all the run operations into a single run operations object
             return get_combined_run_operations(run_operations_list)
 
+    @record_usage_event(LogDatasetEvent)
     def log_inputs(
         self,
         run_id: str,
-        datasets: Optional[list[DatasetInput]] = None,
-        models: Optional[list[LoggedModelInput]] = None,
+        datasets: list[DatasetInput] | None = None,
+        models: list[LoggedModelInput] | None = None,
     ):
         """Log one or more dataset inputs to a run.
 
@@ -678,9 +713,7 @@ class TrackingServiceClient:
 
         return list_artifacts(run_id=run_id, artifact_path=path, tracking_uri=self.tracking_uri)
 
-    def list_logged_model_artifacts(
-        self, model_id: str, path: Optional[str] = None
-    ) -> list[FileInfo]:
+    def list_logged_model_artifacts(self, model_id: str, path: str | None = None) -> list[FileInfo]:
         """List the artifacts for a logged model.
 
         Args:
@@ -693,7 +726,7 @@ class TrackingServiceClient:
         """
         return self._get_artifact_repo(model_id, resource="logged_model").list_artifacts(path)
 
-    def download_artifacts(self, run_id: str, path: str, dst_path: Optional[str] = None):
+    def download_artifacts(self, run_id: str, path: str, dst_path: str | None = None):
         """Download an artifact file or directory from a run to a local directory if applicable,
         and return a local path for it.
 
@@ -725,9 +758,17 @@ class TrackingServiceClient:
         host_url = get_workspace_url()
         if host_url is None:
             host_url = self.store.get_host_creds().host.rstrip("/")
-        run_info = self.store.get_run(run_id).info
-        experiment_id = run_info.experiment_id
-        run_name = run_info.run_name
+        run = self.store.get_run(run_id)
+
+        # Check for a special run tag that indicates the run is triggered by evaluation.
+        # MLflow already shows a link to evaluation results so no need to print it again.
+        if (
+            is_eval_tag := run.data.tags.get(MLFLOW_RUN_IS_EVALUATION)
+        ) and is_eval_tag.lower() == "true":
+            return
+
+        experiment_id = run.info.experiment_id
+        run_name = run.info.run_name
         if is_databricks_uri(self.tracking_uri):
             experiment_url = f"{host_url}/ml/experiments/{experiment_id}"
         else:
@@ -812,14 +853,18 @@ class TrackingServiceClient:
             page_token=page_token,
         )
 
+    @record_usage_event(CreateLoggedModelEvent)
     def create_logged_model(
         self,
         experiment_id: str,
-        name: Optional[str] = None,
-        source_run_id: Optional[str] = None,
-        tags: Optional[dict[str, str]] = None,
-        params: Optional[dict[str, str]] = None,
-        model_type: Optional[str] = None,
+        name: str | None = None,
+        source_run_id: str | None = None,
+        tags: dict[str, str] | None = None,
+        params: dict[str, str] | None = None,
+        model_type: str | None = None,
+        # This parameter is only used for telemetry purposes, and
+        # does not affect the logged model.
+        flavor: str | None = None,
     ) -> LoggedModel:
         return self.store.create_logged_model(
             experiment_id=experiment_id,
@@ -843,6 +888,7 @@ class TrackingServiceClient:
     def finalize_logged_model(self, model_id: str, status: LoggedModelStatus) -> LoggedModel:
         return self.store.finalize_logged_model(model_id, status)
 
+    @record_usage_event(GetLoggedModelEvent)
     def get_logged_model(self, model_id: str) -> LoggedModel:
         return self.store.get_logged_model(model_id)
 
@@ -866,11 +912,11 @@ class TrackingServiceClient:
     def search_logged_models(
         self,
         experiment_ids: list[str],
-        filter_string: Optional[str] = None,
-        datasets: Optional[list[dict[str, Any]]] = None,
-        max_results: Optional[int] = None,
-        order_by: Optional[list[dict[str, Any]]] = None,
-        page_token: Optional[str] = None,
+        filter_string: str | None = None,
+        datasets: list[dict[str, Any]] | None = None,
+        max_results: int | None = None,
+        order_by: list[dict[str, Any]] | None = None,
+        page_token: str | None = None,
     ):
         if not isinstance(experiment_ids, list) or not all(
             isinstance(eid, str) for eid in experiment_ids
@@ -881,3 +927,174 @@ class TrackingServiceClient:
         return self.store.search_logged_models(
             experiment_ids, filter_string, datasets, max_results, order_by, page_token
         )
+
+    @record_usage_event(CreateDatasetEvent)
+    def create_dataset(
+        self,
+        name: str,
+        experiment_id: str | list[str] | None = None,
+        tags: dict[str, Any] | None = None,
+    ) -> "EvaluationDataset":
+        """
+        Create a new dataset.
+
+        Args:
+            name: Name of the dataset.
+            experiment_id: Single experiment ID (str), list of experiment IDs, or None.
+            tags: Dictionary of tags to apply to the dataset.
+
+        Returns:
+            The created EvaluationDataset object.
+        """
+        experiment_ids = [experiment_id] if isinstance(experiment_id, str) else experiment_id
+        context_tags = context_registry.resolve_tags()
+        merged_tags = tags.copy() if tags else {}
+
+        if MLFLOW_USER not in merged_tags and MLFLOW_USER in context_tags:
+            merged_tags[MLFLOW_USER] = context_tags[MLFLOW_USER]
+
+        return self.store.create_dataset(
+            name=name,
+            tags=merged_tags if merged_tags else None,
+            experiment_ids=experiment_ids,
+        )
+
+    def get_dataset(self, dataset_id: str) -> "EvaluationDataset":
+        """
+        Get a dataset by ID.
+
+        Args:
+            dataset_id: ID of the dataset to retrieve.
+
+        Returns:
+            The EvaluationDataset object.
+        """
+        return self.store.get_dataset(dataset_id)
+
+    def delete_dataset(self, dataset_id: str) -> None:
+        """
+        Delete a dataset.
+
+        Args:
+            dataset_id: ID of the dataset to delete.
+        """
+        self.store.delete_dataset(dataset_id)
+
+    def search_datasets(
+        self,
+        experiment_ids: list[str] | None = None,
+        filter_string: str | None = None,
+        max_results: int = 1000,
+        order_by: list[str] | None = None,
+        page_token: str | None = None,
+    ) -> PagedList["EvaluationDataset"]:
+        """
+        Search for datasets.
+
+        Args:
+            experiment_ids: List of experiment IDs to filter by.
+            filter_string: Filter query string.
+            max_results: Maximum number of datasets to return.
+            order_by: List of columns to order by.
+            page_token: Token for retrieving the next page of results.
+
+        Returns:
+            A PagedList of EvaluationDataset objects.
+        """
+        return self.store.search_datasets(
+            experiment_ids=experiment_ids,
+            filter_string=filter_string,
+            max_results=max_results,
+            order_by=order_by,
+            page_token=page_token,
+        )
+
+    def set_dataset_tags(self, dataset_id: str, tags: dict[str, Any]) -> None:
+        """
+        Set tags for a dataset.
+
+        This implements an upsert operation - existing tags are merged with new tags.
+        To remove a tag, set its value to None.
+
+        Args:
+            dataset_id: The ID of the dataset to update.
+            tags: Dictionary of tags to update. Setting a value to None removes the tag.
+
+        Raises:
+            MlflowException: If dataset not found or invalid parameters.
+        """
+        self.store.set_dataset_tags(dataset_id=dataset_id, tags=tags)
+
+    def delete_dataset_tag(self, dataset_id: str, key: str) -> None:
+        """
+        Delete a tag from a dataset.
+
+        Args:
+            dataset_id: The ID of the dataset.
+            key: The tag key to delete.
+
+        Raises:
+            MlflowException: If dataset not found.
+        """
+        self.store.delete_dataset_tag(dataset_id=dataset_id, key=key)
+
+    def add_dataset_to_experiments(
+        self, dataset_id: str, experiment_ids: list[str]
+    ) -> "EvaluationDataset":
+        """
+        Add a dataset to additional experiments.
+
+        Args:
+            dataset_id: The ID of the dataset to update.
+            experiment_ids: List of experiment IDs to associate with the dataset.
+
+        Returns:
+            The updated EvaluationDataset with new experiment associations.
+
+        Raises:
+            MlflowException: If dataset or experiments not found.
+        """
+        return self.store.add_dataset_to_experiments(dataset_id, experiment_ids)
+
+    def remove_dataset_from_experiments(
+        self, dataset_id: str, experiment_ids: list[str]
+    ) -> "EvaluationDataset":
+        """
+        Remove a dataset from experiments.
+
+        Args:
+            dataset_id: The ID of the dataset to update.
+            experiment_ids: List of experiment IDs to remove association from.
+
+        Returns:
+            The updated EvaluationDataset with removed experiment associations.
+
+        Raises:
+            MlflowException: If dataset not found.
+        """
+        return self.store.remove_dataset_from_experiments(dataset_id, experiment_ids)
+
+    def link_traces_to_run(self, trace_ids: list[str], run_id: str) -> None:
+        """
+        Link multiple traces to a run by creating entity associations.
+
+        Args:
+            trace_ids: List of trace IDs to link to the run. Maximum 100 traces allowed.
+            run_id: ID of the run to link traces to.
+
+        Raises:
+            MlflowException: If more than 100 traces are provided or run_id is empty.
+        """
+        if not trace_ids:
+            return
+
+        if not run_id:
+            raise MlflowException.invalid_parameter_value("run_id cannot be empty")
+
+        if len(trace_ids) > 100:
+            raise MlflowException.invalid_parameter_value(
+                f"Cannot link more than 100 traces to a run in a single request. "
+                f"Provided {len(trace_ids)} traces."
+            )
+
+        return self.store.link_traces_to_run(trace_ids, run_id)
