@@ -7,7 +7,7 @@ import platform
 import subprocess
 import time
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, NamedTuple, Optional, TypeVar
+from typing import TYPE_CHECKING, Callable, NamedTuple, ParamSpec, TypeVar
 
 from mlflow.utils.logging_utils import eprint
 from mlflow.utils.request_utils import augmented_raise_for_status
@@ -30,6 +30,7 @@ from mlflow.legacy_databricks_cli.configure.provider import (
     ProfileConfigProvider,
     SparkTaskContextConfigProvider,
 )
+from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
 from mlflow.utils._spark_utils import _get_active_spark_session
 from mlflow.utils.rest_utils import MlflowHostCreds, http_request
 from mlflow.utils.uri import (
@@ -313,7 +314,7 @@ class DBConnectUDFSandboxInfo:
     mlflow_version: str
 
 
-_dbconnect_udf_sandbox_info_cache: Optional[DBConnectUDFSandboxInfo] = None
+_dbconnect_udf_sandbox_info_cache: DBConnectUDFSandboxInfo | None = None
 
 
 def get_dbconnect_udf_sandbox_info(spark):
@@ -905,7 +906,7 @@ def is_running_in_ipython_environment():
         return False
 
 
-def get_databricks_run_url(tracking_uri: str, run_id: str, artifact_path=None) -> Optional[str]:
+def get_databricks_run_url(tracking_uri: str, run_id: str, artifact_path=None) -> str | None:
     """
     Obtains a Databricks URL corresponding to the specified MLflow Run, optionally referring
     to an artifact within the run.
@@ -941,7 +942,7 @@ def get_databricks_run_url(tracking_uri: str, run_id: str, artifact_path=None) -
         return None
 
 
-def get_databricks_model_version_url(registry_uri: str, name: str, version: str) -> Optional[str]:
+def get_databricks_model_version_url(registry_uri: str, name: str, version: str) -> str | None:
     """Obtains a Databricks URL corresponding to the specified Model Version.
 
     Args:
@@ -977,12 +978,12 @@ class DatabricksWorkspaceInfo:
     WORKSPACE_HOST_ENV_VAR = "_DATABRICKS_WORKSPACE_HOST"
     WORKSPACE_ID_ENV_VAR = "_DATABRICKS_WORKSPACE_ID"
 
-    def __init__(self, host: str, workspace_id: Optional[str] = None):
+    def __init__(self, host: str, workspace_id: str | None = None):
         self.host = host
         self.workspace_id = workspace_id
 
     @classmethod
-    def from_environment(cls) -> Optional[DatabricksWorkspaceInfoType]:
+    def from_environment(cls) -> DatabricksWorkspaceInfoType | None:
         if DatabricksWorkspaceInfo.WORKSPACE_HOST_ENV_VAR in os.environ:
             return DatabricksWorkspaceInfo(
                 host=os.environ[DatabricksWorkspaceInfo.WORKSPACE_HOST_ENV_VAR],
@@ -1001,7 +1002,7 @@ class DatabricksWorkspaceInfo:
         return env
 
 
-def get_databricks_workspace_info_from_uri(tracking_uri: str) -> Optional[DatabricksWorkspaceInfo]:
+def get_databricks_workspace_info_from_uri(tracking_uri: str) -> DatabricksWorkspaceInfo | None:
     if not is_databricks_uri(tracking_uri):
         return None
 
@@ -1046,8 +1047,8 @@ def _construct_databricks_run_url(
     host: str,
     experiment_id: str,
     run_id: str,
-    workspace_id: Optional[str] = None,
-    artifact_path: Optional[str] = None,
+    workspace_id: str | None = None,
+    artifact_path: str | None = None,
 ) -> str:
     run_url = host
     if workspace_id and workspace_id != "0":
@@ -1062,7 +1063,7 @@ def _construct_databricks_run_url(
 
 
 def _construct_databricks_model_version_url(
-    host: str, name: str, version: str, workspace_id: Optional[str] = None
+    host: str, name: str, version: str, workspace_id: str | None = None
 ) -> str:
     model_version_url = host
     if workspace_id and workspace_id != "0":
@@ -1074,7 +1075,7 @@ def _construct_databricks_model_version_url(
 
 
 def _construct_databricks_logged_model_url(
-    workspace_url: str, experiment_id: str, model_id: str, workspace_id: Optional[str] = None
+    workspace_url: str, experiment_id: str, model_id: str, workspace_id: str | None = None
 ) -> str:
     """
     Get a Databricks URL for a given registered model version in Unity Catalog.
@@ -1093,7 +1094,7 @@ def _construct_databricks_logged_model_url(
 
 
 def _construct_databricks_uc_registered_model_url(
-    workspace_url: str, registered_model_name: str, version: str, workspace_id: Optional[str] = None
+    workspace_url: str, registered_model_name: str, version: str, workspace_id: str | None = None
 ) -> str:
     """
     Get a Databricks URL for a given registered model version in Unity Catalog.
@@ -1115,8 +1116,8 @@ def _construct_databricks_uc_registered_model_url(
 def _print_databricks_deployment_job_url(
     model_name: str,
     job_id: str,
-    workspace_url: Optional[str] = None,
-    workspace_id: Optional[str] = None,
+    workspace_url: str | None = None,
+    workspace_id: str | None = None,
 ) -> str:
     if not workspace_url:
         workspace_url = get_workspace_url()
@@ -1242,7 +1243,7 @@ class DatabricksRuntimeVersion(NamedTuple):
     minor: int
 
     @classmethod
-    def parse(cls, databricks_runtime: Optional[str] = None):
+    def parse(cls, databricks_runtime: str | None = None):
         dbr_version = databricks_runtime or get_databricks_runtime_version()
         try:
             dbr_version_splits = dbr_version.split(".", maxsplit=2)
@@ -1434,3 +1435,45 @@ def stage_model_for_databricks_model_serving(model_name: str, model_version: str
         },
     )
     augmented_raise_for_status(response)
+
+
+P = ParamSpec("P")
+T = TypeVar("T")
+
+
+def databricks_api_disabled(api_name: str = "This API", alternative: str | None = None):
+    """
+    Decorator that disables an API method when used with Databricks.
+
+    This decorator checks if the tracking URI is a Databricks URI and raises an error if so.
+
+    Args:
+        api_name: Name of the API for the error message.
+        alternative: Optional alternative solution to suggest in the error message.
+
+    Returns:
+        Decorator function that wraps the method to check for Databricks.
+    """
+
+    def decorator(func: Callable[P, T]) -> Callable[P, T]:
+        @functools.wraps(func)
+        def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
+            from mlflow.tracking import get_tracking_uri
+            from mlflow.utils.uri import is_databricks_uri
+
+            tracking_uri = get_tracking_uri()
+            if not is_databricks_uri(tracking_uri):
+                return func(*args, **kwargs)
+
+            error_msg = f"{api_name} is not supported in Databricks environments."
+            if alternative:
+                error_msg += f" {alternative}"
+
+            raise MlflowException(
+                error_msg,
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+
+        return wrapper
+
+    return decorator
