@@ -5,7 +5,7 @@ import random
 import time
 import warnings
 from functools import lru_cache
-from typing import Callable
+from typing import Any, Callable
 
 import requests
 
@@ -92,7 +92,7 @@ def http_request(
             in retry_codes range and retries have been exhausted.
         respect_retry_after_header: Whether to respect Retry-After header on status codes defined
             as Retry.RETRY_AFTER_STATUS_CODES or not.
-        retry_timeout_seconds: Timeout for retires. Only effective when using Databricks SDK.
+        retry_timeout_seconds: Timeout for retries. Only effective when using Databricks SDK.
         kwargs: Additional keyword arguments to pass to `requests.Session.request()`
 
     Returns:
@@ -110,6 +110,12 @@ def http_request(
         MLFLOW_HTTP_REQUEST_BACKOFF_JITTER.get() if backoff_jitter is None else backoff_jitter
     )
 
+    from mlflow.tracking.request_header.registry import resolve_request_headers
+
+    headers = dict(**resolve_request_headers())
+    if extra_headers:
+        headers = dict(**headers, **extra_headers)
+
     if host_creds.use_databricks_sdk:
         from databricks.sdk.errors import DatabricksError
 
@@ -134,7 +140,7 @@ def http_request(
                 raw_response = ws_client.api_client.do(
                     method=method,
                     path=endpoint,
-                    headers=extra_headers,
+                    headers=headers,
                     raw=True,
                     query=kwargs.get("params"),
                     body=kwargs.get("json"),
@@ -204,12 +210,6 @@ def http_request(
             error_code=CUSTOMER_UNAUTHORIZED,
         )
 
-    from mlflow.tracking.request_header.registry import resolve_request_headers
-
-    headers = dict(**resolve_request_headers())
-    if extra_headers:
-        headers = dict(**headers, **extra_headers)
-
     if auth_str:
         headers["Authorization"] = auth_str
 
@@ -377,9 +377,18 @@ def _validate_backoff_factor(backoff_factor):
         )
 
 
+def _time_sleep(seconds: float) -> None:
+    """
+    This function is specifically mocked in `test_rest_utils.py` to test the backoff logic in
+    isolation. We avoid wrapping `time.sleep` globally to prevent interfering with unrelated sleep
+    calls elsewhere in the codebase or in third-party libraries.
+    """
+    time.sleep(seconds)
+
+
 def _retry_databricks_sdk_call_with_exponential_backoff(
     *,
-    call_func: Callable,
+    call_func: Callable[..., Any],
     retry_codes: list[int],
     retry_timeout_seconds: int,
     backoff_factor: int,
@@ -403,7 +412,7 @@ def _retry_databricks_sdk_call_with_exponential_backoff(
     Raises:
         DatabricksError: If all retries are exhausted or non-retryable error occurs
     """
-    from databricks.sdk.errors import DatabricksError
+    from databricks.sdk.errors import STATUS_CODE_MAPPING, DatabricksError
 
     start_time = time.time()
     attempt = 0
@@ -413,8 +422,9 @@ def _retry_databricks_sdk_call_with_exponential_backoff(
             return call_func()
         except DatabricksError as e:
             # Get HTTP status code from the error
-            status_code = ERROR_CODE_TO_HTTP_STATUS.get(e.error_code, 500)
-
+            status_code = next(
+                (code for code, cls in STATUS_CODE_MAPPING.items() if isinstance(e, cls)), 500
+            )
             # Check if this is a retryable error
             if status_code not in retry_codes:
                 raise
@@ -446,7 +456,7 @@ def _retry_databricks_sdk_call_with_exponential_backoff(
                 f"Retrying in {backoff_time:.2f} seconds (attempt {attempt + 1})"
             )
 
-            time.sleep(backoff_time)
+            _time_sleep(backoff_time)
             attempt += 1
 
 
@@ -478,7 +488,7 @@ def extract_all_api_info_for_service(service, path_prefix):
     return res
 
 
-def get_single_trace_endpoint(request_id, use_v3=False):
+def get_single_trace_endpoint(request_id, use_v3=True):
     """
     Get the endpoint for a single trace.
     For Databricks tracking URIs, use the V3 API.
@@ -497,70 +507,20 @@ def get_logged_model_endpoint(model_id: str) -> str:
     return f"{_REST_API_PATH_PREFIX}/mlflow/logged-models/{model_id}"
 
 
-def get_trace_info_endpoint(request_id):
-    return f"{_TRACE_REST_API_PATH_PREFIX}/{request_id}/info"
-
-
-def get_trace_assessment_endpoint(request_id, is_databricks=False):
-    """
-    Get the endpoint for a trace assessment.
-
-    Args:
-        request_id: The trace ID.
-        is_databricks: Whether the tracking URI is a Databricks URI.
-    """
-    return get_single_trace_endpoint(request_id, use_v3=is_databricks)
-
-
-def get_set_trace_tag_endpoint(request_id, is_databricks=False):
-    """
-    Get the endpoint for setting trace tags.
-
-    Args:
-        request_id: The trace ID.
-        is_databricks: Whether the tracking URI is a Databricks URI.
-    """
-    return f"{get_single_trace_endpoint(request_id, use_v3=is_databricks)}/tags"
-
-
-def get_create_assessment_endpoint(trace_id: str, is_databricks=False):
-    """
-    Get the endpoint for creating an assessment.
-
-    Args:
-        trace_id: The trace ID.
-        is_databricks: Whether the tracking URI is a Databricks URI.
-    """
-    if is_databricks:
-        return f"{_V3_TRACE_REST_API_PATH_PREFIX}/{trace_id}/assessments"
-    return f"{_TRACE_REST_API_PATH_PREFIX}/{trace_id}/assessments"
-
-
-def get_single_assessment_endpoint(trace_id: str, assessment_id: str, is_databricks=False):
+def get_single_assessment_endpoint(trace_id: str, assessment_id: str) -> str:
     """
     Get the endpoint for a single assessment.
 
     Args:
         trace_id: The trace ID.
         assessment_id: The assessment ID.
-        is_databricks: Whether the tracking URI is a Databricks URI.
     """
-    if is_databricks:
-        return f"{_V3_TRACE_REST_API_PATH_PREFIX}/{trace_id}/assessments/{assessment_id}"
-    return f"{_TRACE_REST_API_PATH_PREFIX}/{trace_id}/assessments/{assessment_id}"
+    return f"{_V3_TRACE_REST_API_PATH_PREFIX}/{trace_id}/assessments/{assessment_id}"
 
 
-def get_search_traces_v3_endpoint(is_databricks=False):
-    """
-    Return the endpoint for the SearchTraces API.
-
-    Args:
-        is_databricks: Whether the tracking URI is a Databricks URI. If True,
-                       returns the v3 endpoint, otherwise returns the v2 endpoint.
-    """
-    if is_databricks:
-        return f"{_V3_TRACE_REST_API_PATH_PREFIX}/search"
-    return f"{_TRACE_REST_API_PATH_PREFIX}/search"
+def get_trace_tag_endpoint(trace_id):
+    """Get the endpoint for trace tags. Always use v2 endpoint."""
+    return f"{_REST_API_PATH_PREFIX}/mlflow/traces/{trace_id}/tags"
 
 
 def call_endpoint(
@@ -707,6 +667,9 @@ class MlflowHostCreds:
             return self.__dict__ == other.__dict__
         return NotImplemented
 
+    def __hash__(self):
+        return hash(frozenset(self.__dict__.items()))
+
     @property
     def verify(self):
         if self.use_databricks_sdk:
