@@ -6,7 +6,7 @@ import operator
 import re
 import shlex
 from dataclasses import asdict, dataclass
-from typing import Any, Optional
+from typing import Any
 
 import sqlparse
 from packaging.version import Version
@@ -1607,10 +1607,12 @@ class SearchTraceUtils(SearchUtils):
     # decision if we find a way to support them efficiently.
     VALID_TAG_COMPARATORS = {"!=", "="}
     VALID_STRING_ATTRIBUTE_COMPARATORS = {"!=", "=", "IN", "NOT IN"}
+    VALID_SPAN_ATTRIBUTE_COMPARATORS = {"!=", "=", "IN", "NOT IN", "LIKE", "ILIKE"}
 
     _REQUEST_METADATA_IDENTIFIER = "request_metadata"
     _TAG_IDENTIFIER = "tag"
     _ATTRIBUTE_IDENTIFIER = "attribute"
+    _SPAN_IDENTIFIER = "span"
 
     # These are aliases for the base identifiers
     # e.g. trace.status is equivalent to attribute.status
@@ -1620,9 +1622,17 @@ class SearchTraceUtils(SearchUtils):
         "trace": _ATTRIBUTE_IDENTIFIER,
         "metadata": _REQUEST_METADATA_IDENTIFIER,
     }
-    _IDENTIFIERS = {_TAG_IDENTIFIER, _REQUEST_METADATA_IDENTIFIER, _ATTRIBUTE_IDENTIFIER}
+    _IDENTIFIERS = {
+        _TAG_IDENTIFIER,
+        _REQUEST_METADATA_IDENTIFIER,
+        _ATTRIBUTE_IDENTIFIER,
+        _SPAN_IDENTIFIER,
+    }
     _VALID_IDENTIFIERS = _IDENTIFIERS | set(_ALTERNATE_IDENTIFIERS.keys())
 
+    # Supported span attributes
+    _SUPPORTED_SPAN_ATTRIBUTES = {"name"}
+
     SUPPORT_IN_COMPARISON_ATTRIBUTE_KEYS = {"name", "status", "request_id", "run_id"}
 
     # Some search keys are defined differently in the DB models.
@@ -1664,6 +1674,12 @@ class SearchTraceUtils(SearchUtils):
             lhs = trace.request_metadata.get(key)
         elif cls.is_attribute(type_, key, comparator):
             lhs = getattr(trace, key)
+        elif cls.is_span(type_, key, comparator):
+            raise MlflowException(
+                "Span filtering requires database support and cannot be performed "
+                "on in-memory trace data.",
+                error_code=INVALID_PARAMETER_VALUE,
+            )
         elif sed.get("type") == cls._TAG_IDENTIFIER:
             lhs = trace.tags.get(key)
         else:
@@ -1697,6 +1713,10 @@ class SearchTraceUtils(SearchUtils):
         """
         Replace search key to tag or metadata key if it is in the mapping.
         """
+        # Don't replace keys for span filters - they have their own namespace
+        if parsed.get("type") == cls._SPAN_IDENTIFIER:
+            return parsed
+
         key = parsed.get("key").lower()
         if key in cls.SEARCH_KEY_TO_TAG:
             parsed["type"] = cls._TAG_IDENTIFIER
@@ -1720,6 +1740,26 @@ class SearchTraceUtils(SearchUtils):
             return True
         return False
 
+    @classmethod
+    def is_span(cls, key_type, key_name, comparator):
+        if key_type == cls._SPAN_IDENTIFIER:
+            if key_name in cls._SUPPORTED_SPAN_ATTRIBUTES:
+                if comparator not in cls.VALID_SPAN_ATTRIBUTE_COMPARATORS:
+                    raise MlflowException(
+                        f"span.{key_name} comparator '{comparator}' not one of "
+                        f"'{cls.VALID_SPAN_ATTRIBUTE_COMPARATORS}'",
+                        error_code=INVALID_PARAMETER_VALUE,
+                    )
+            else:
+                supported_attrs = ", ".join(sorted(cls._SUPPORTED_SPAN_ATTRIBUTES))
+                raise MlflowException(
+                    f"Invalid span attribute '{key_name}'. "
+                    f"Supported attributes: {supported_attrs}.",
+                    error_code=INVALID_PARAMETER_VALUE,
+                )
+            return True
+        return False
+
     @classmethod
     def _valid_entity_type(cls, entity_type):
         entity_type = cls._trim_backticks(entity_type)
@@ -1804,6 +1844,18 @@ class SearchTraceUtils(SearchUtils):
                     f"{token.value}",
                     error_code=INVALID_PARAMETER_VALUE,
                 )
+        elif identifier_type == cls._SPAN_IDENTIFIER:
+            if token.ttype in cls.STRING_VALUE_TYPES or isinstance(token, Identifier):
+                return cls._strip_quotes(token.value, expect_quoted_value=True)
+            elif isinstance(token, Parenthesis):
+                return cls._parse_attribute_lists(token)
+            else:
+                raise MlflowException(
+                    "Expected a quoted string value for "
+                    f"{identifier_type} (e.g. 'my-value'). Got value "
+                    f"{token.value}",
+                    error_code=INVALID_PARAMETER_VALUE,
+                )
         else:
             # Expected to be either "param" or "metric".
             raise MlflowException(
@@ -1846,9 +1898,91 @@ class SearchTraceUtils(SearchUtils):
         comp = cls._get_identifier(stripped_comparison[0].value, cls.VALID_SEARCH_ATTRIBUTE_KEYS)
         comp["comparator"] = stripped_comparison[1].value
         comp["value"] = cls._get_value(comp.get("type"), comp.get("key"), stripped_comparison[2])
+
+        if comp.get("type") == cls._SPAN_IDENTIFIER:
+            cls.is_span(comp["type"], comp["key"], comp["comparator"])
+
         return comp
 
 
+class SearchEvaluationDatasetsUtils(SearchUtils):
+    """
+    Utility class for searching evaluation datasets.
+    """
+
+    VALID_SEARCH_ATTRIBUTE_KEYS = {
+        "name",
+        "created_time",
+        "last_update_time",
+        "created_by",
+        "last_updated_by",
+    }
+    VALID_ORDER_BY_ATTRIBUTE_KEYS = {"name", "created_time", "last_update_time"}
+    NUMERIC_ATTRIBUTES = {"created_time", "last_update_time"}
+
+    @classmethod
+    def _invalid_statement_token(cls, token):
+        if (
+            isinstance(token, Comparison)
+            or token.is_whitespace
+            or token.match(ttype=TokenType.Keyword, values=["AND"])
+        ):
+            return False
+        return True
+
+    @classmethod
+    def _process_statement(cls, statement):
+        tokens = _join_in_comparison_tokens(statement.tokens)
+        invalids = list(filter(cls._invalid_statement_token, tokens))
+        if len(invalids) > 0:
+            invalid_clauses = ", ".join(map(str, invalids))
+            raise MlflowException.invalid_parameter_value(
+                f"Invalid clause(s) in filter string: {invalid_clauses}"
+            )
+        return [cls._get_comparison(t) for t in tokens if isinstance(t, Comparison)]
+
+    @classmethod
+    def _get_identifier(cls, identifier, valid_attributes):
+        tokens = identifier.split(".", maxsplit=1)
+        if len(tokens) == 1:
+            key = tokens[0]
+            if key not in valid_attributes:
+                raise MlflowException.invalid_parameter_value(
+                    f"Invalid attribute key '{key}' specified. Valid keys are: {valid_attributes}"
+                )
+            return {"type": "attribute", "key": key}
+        else:
+            if tokens[0] == "tags":
+                key = tokens[1]
+                return {"type": "tag", "key": key}
+            else:
+                raise MlflowException.invalid_parameter_value(
+                    f"Invalid identifier token '{tokens[0]}' specified"
+                )
+
+    @classmethod
+    def parse_order_by_for_search_evaluation_datasets(cls, order_by):
+        token_value, is_ascending = cls._parse_order_by_string(order_by)
+        identifier = cls._get_identifier(token_value.strip(), cls.VALID_ORDER_BY_ATTRIBUTE_KEYS)
+        return identifier["type"], identifier["key"], is_ascending
+
+    @classmethod
+    def is_string_attribute(cls, type_, key, comparator):
+        return (
+            type_ == "attribute"
+            and key not in cls.NUMERIC_ATTRIBUTES
+            and comparator in cls.VALID_STRING_ATTRIBUTE_COMPARATORS
+        )
+
+    @classmethod
+    def is_numeric_attribute(cls, type_, key, comparator):
+        return (
+            type_ == "attribute"
+            and key in cls.NUMERIC_ATTRIBUTES
+            and comparator in cls.VALID_NUMERIC_ATTRIBUTE_COMPARATORS
+        )
+
+
 class SearchLoggedModelsUtils(SearchUtils):
     NUMERIC_ATTRIBUTES = {
         "creation_timestamp",
@@ -1870,7 +2004,7 @@ class SearchLoggedModelsUtils(SearchUtils):
         cls,
         model: LoggedModel,
         condition: dict[str, Any],
-        datasets: Optional[list[dict[str, Any]]] = None,
+        datasets: list[dict[str, Any]] | None = None,
     ):
         key_type = condition.get("type")
         key = condition.get("key")
@@ -1917,8 +2051,8 @@ class SearchLoggedModelsUtils(SearchUtils):
     def filter_logged_models(
         cls,
         models: list[LoggedModel],
-        filter_string: Optional[str] = None,
-        datasets: Optional[list[dict[str, Any]]] = None,
+        filter_string: str | None = None,
+        datasets: list[dict[str, Any]] | None = None,
     ):
         """Filters a set of runs based on a search filter string and list of dataset filters."""
         if not filter_string and not datasets:
@@ -1949,8 +2083,8 @@ class SearchLoggedModelsUtils(SearchUtils):
     class OrderBy:
         field_name: str
         ascending: bool = True
-        dataset_name: Optional[str] = None
-        dataset_digest: Optional[str] = None
+        dataset_name: str | None = None
+        dataset_digest: str | None = None
 
     @classmethod
     def parse_order_by_for_logged_models(cls, order_by: dict[str, Any]) -> OrderBy:
@@ -2025,7 +2159,7 @@ class SearchLoggedModelsUtils(SearchUtils):
         return value if order_by.ascending else _Reversor(value)
 
     @classmethod
-    def _get_sort_key(cls, order_by_list: Optional[list[dict[str, Any]]]):
+    def _get_sort_key(cls, order_by_list: list[dict[str, Any]] | None):
         parsed_order_by = list(map(cls.parse_order_by_for_logged_models, order_by_list or []))
 
         # Add a tie-breaker
@@ -2062,8 +2196,8 @@ class _LoggedModelMetricComp:
 @dataclass
 class SearchLoggedModelsPaginationToken:
     experiment_ids: list[str]
-    filter_string: Optional[str] = None
-    order_by: Optional[list[dict[str, Any]]] = None
+    filter_string: str | None = None
+    order_by: list[dict[str, Any]] | None = None
     offset: int = 0
 
     def to_json(self) -> str:
@@ -2089,8 +2223,8 @@ class SearchLoggedModelsPaginationToken:
     def validate(
         self,
         experiment_ids: list[str],
-        filter_string: Optional[str],
-        order_by: Optional[list[dict[str, Any]]],
+        filter_string: str | None,
+        order_by: list[dict[str, Any]] | None,
     ) -> None:
         if self.experiment_ids != experiment_ids:
             raise MlflowException.invalid_parameter_value(
